// SocketReader.java
//package jwiley.chp9;

import java.net.DatagramSocket;
import java.net.Socket;
import java.io.InputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

/** Class to demonstrate reading from a Socket. */
public class DatagramReader
{
    /** main() method to invoke from JVM. */
    public static void main(String args[])
    {
         if (args.length < 1)
         {
            System.out.println("USAGE: java DatagramReader port");
            System.exit(1);
         }

         String portStr = new String(args[0]);

         try
         {
            int port = Integer.parseInt(portStr);

            // create the server socket
	    DatagramSocket readServer = new DatagramSocket(port);
//            ServerSocket readServer = new ServerSocket(port);
            System.out.println("Datagram Socket created to " + port);
            System.out.println("Awaiting a connection for 600 seconds...");

            // set socket timeout
            readServer.setSoTimeout(600000);     // 600 seconds

            // wait for a connection
            Socket connection = null;
            try
            {
               connection = readServer.accept();
               System.out.println("Connection accepted.");
            } catch (IOException ioe)
              {
               System.out.println("Socket reader timed out.");
               System.exit(1);
              }

            // set socket timeout
            connection.setSoTimeout(10000);     // 10 seconds

            // set up the streams
            InputStream inStream = connection.getInputStream();
            BufferedReader br = new BufferedReader(new InputStreamReader(inStream));

            // while input does not equal 'exit' output to socket
            boolean done = false;
            int timeout = 0;
            while (!done)
            {
                try
                {
                   String line = br.readLine();
                   if (line != null && !line.startsWith("~READER_STOP~"))
                   {
                      System.out.println(line);
                   }
                   else
                      done = true;
                } catch (IOException ioe)
                  {
                   System.out.println("Socket read timed out.");
                   if(timeout++ > 1000)done = true;
                  }
            }

            // close the server
            readServer.close();
         } catch (Exception e)
           {
            e.printStackTrace();
            System.exit(1);
           }
    }
}
