package com.nokia.m2m.sdk.examples.device;

import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import org.omg.CORBA.StringHolder;

import com.nokia.m2m.orb.idl.wirelessdevice.Device;
import com.nokia.m2m.orb.idl.wirelessdevice.DeviceHelper;
import com.nokia.m2m.orb.idl.wirelessdevice.DevicePackage.MetadataListHolder;
import com.nokia.m2m.orb.idl.wirelessdevice.DevicePackage.VersionHolder;
import com.nokia.m2m.sdk.examples.logger.SerialPortLogger;

/**
 * 
 * IMlet asks device information from wireless device and
 * prints returned information to serial port. 
 * 
 */
public class DeviceInfo extends MIDlet {

	protected void startApp() throws MIDletStateChangeException {

		org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(null, null);

		//
		// Resolve the Device object.
		//
		String url = "corbaloc::127.0.0.1:19740/Device";
		org.omg.CORBA.Object ref = orb.string_to_object(url);
		Device device = DeviceHelper.narrow(ref);

		/*
		  Related IDL goes as follows:
		 
			void getDeviceInfo(
				out string	hwManufacturer,
				out string	hwProductName,
				out Version	hwVersion,
				out string	swManufacturer,
				out string	swProductName,
				out Version	swVersion,
				out MetadataList metadata );
		
		*/

		//
		// Create holders for 'out' arguments.
		//
		StringHolder hwManufacturer = new StringHolder();
		StringHolder hwProductName = new StringHolder();
		VersionHolder hwVersion = new VersionHolder();

		StringHolder swManufacturer = new StringHolder();
		StringHolder swProductName = new StringHolder();
		VersionHolder swVersion = new VersionHolder();

		MetadataListHolder metadata = new MetadataListHolder();

		//
		// Invoke the method.
		//
		device.getDeviceInfo(
			hwManufacturer,
			hwProductName,
			hwVersion,
			swManufacturer,
			swProductName,
			swVersion,
			metadata);

		//
		// Print the results.
		//
		String deviceInfo = "hwManufacturer=" + hwManufacturer.value + "\n";
		deviceInfo += "hwProductName=" + hwProductName.value + "\n";
		deviceInfo += "hwVersion="
			+ hwVersion.value.major
			+ "."
			+ hwVersion.value.minor
			+ " build "
			+ hwVersion.value.build
			+ "\n";

		deviceInfo += "swManufacturer=" + swManufacturer.value + "\n";
		deviceInfo += "swProductName=" + swProductName.value + "\n";
		deviceInfo += "swVersion="
			+ swVersion.value.major
			+ "."
			+ swVersion.value.minor
			+ " build "
			+ swVersion.value.build
			+ "\n";

		log(deviceInfo);

	}

	/**
	 * Print message to the serial port
	 * @param msg message to be printed
	 */
	private void log(String msg) {
		SerialPortLogger.getInstance().write(msg);
	}

	protected void pauseApp() {
	}

	protected void destroyApp(boolean arg0) throws MIDletStateChangeException {
	}

}
