package com.nokia.m2m.sdk.examples.device;

import java.util.Hashtable;

import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;
import org.omg.CORBA.Policy;
import org.omg.PortableServer.IdAssignmentPolicyValue;
import org.omg.PortableServer.IdUniquenessPolicyValue;
import org.omg.PortableServer.LifespanPolicyValue;
import org.omg.PortableServer.POAHelper;

import org.omg.PortableServer.POA;

import com.nokia.m2m.orb.idl.terminal.ET;
import com.nokia.m2m.orb.idl.terminal.ETHelper;
import com.nokia.m2m.orb.idl.wirelessdevice.Device;
import com.nokia.m2m.orb.idl.wirelessdevice.DeviceHelper;
import com.nokia.m2m.orb.idl.wirelessdevice.EventObserverHelper;
import com.nokia.m2m.sdk.examples.logger.SerialPortLogger;

/**
 * MessageObserverIMlet listens for incoming short messages and sends
 * a response message back to the sender. This implementation uses
 * event listener approach.
 *
 *
 */
public class MessageObserverIMlet extends MIDlet {

	ORB orb;

	/**
	 * Start application
	 */
	protected void startApp() throws MIDletStateChangeException {

		try {
			log("MessageObserverIMlet starting");

			// Initialize ORB in IIOP mode. This IMlet does not do wireless
			// communications.
			Hashtable props = new Hashtable();
			props.put("com.nokia.m2m.orb.UseM2MGateway", "no");
			orb = ORB.init(null, props);

			//
			// Get the RootPOA.
			//
			org.omg.PortableServer.POA rootPoa;
			rootPoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));

			//
			// Create a new POA. Use persistent policies
			// so that servants activated under
			// this POA will always get the same IOR. This ensures
			// that when the IMlet is restarted
			// after Nokia 12 module reboot, the new reqistration will
			// overwrite the existing reqistration.
			//
			Policy[] policies =
				new Policy[] {
					rootPoa.create_id_assignment_policy(IdAssignmentPolicyValue.USER_ID),
					rootPoa.create_id_uniqueness_policy(IdUniquenessPolicyValue.UNIQUE_ID),
					rootPoa.create_lifespan_policy(LifespanPolicyValue.PERSISTENT)};

			//
			// Name the new POA as 'SmsObserverPOA' this name will be part of
			// the listener objects object key. Object key can be freely chosen.
			//
			POA observerPoa = rootPoa.create_POA("SmsObserverPOA", rootPoa.the_POAManager(), policies);

			// Resolve the Embedded Terminal object. This is needed to send response shortmessages.
			String etUrl =  "corbaloc::127.0.0.1:19740/ORB/OA/IDL:ET:1.0";
			org.omg.CORBA.Object etRef = orb.string_to_object(etUrl);
			ET et = ETHelper.narrow(etRef);

			String deviceUrl =  "corbaloc::127.0.0.1:19740/Device";
			org.omg.CORBA.Object deviceRef = orb.string_to_object(deviceUrl);
			Device device=DeviceHelper.narrow(deviceRef);

			//
			// Initialize the servant object. Give embedded terminal to the observer so that it can
			// use et methods to send response shortmessage back to the sender. Object id can be freely
			// chosen.
			//
			MessageObserverImp observerImpl = new MessageObserverImp(et);
			observerPoa.activate_object_with_id("SMSObserver".getBytes(), observerImpl);

			//
			// Activate the POAManager.
			// Activation starts request processing in all POAs that
			// use this POAManager.
			//
			rootPoa.the_POAManager().activate();

			//
			// The observer is now ready to receive events.
			// Register the listener into the Module ORB.
			//

			// Get CORBA object reference from servant
			Object observerObj = observerPoa.servant_to_reference(observerImpl);

			log("Setting observer");

			// Start to listen to the SignalQuality parameter.
			device.setEventObserver("SMS-RECEIVED-TEXT", EventObserverHelper.narrow(observerObj));

			// Activate also SMS-RECEIVED-UNRESOLVED if you want to receive binary short messages.
			// Same observer object can be set to listen multiple different events
			device.setEventObserver("SMS-RECEIVED-UNRESOLVED", EventObserverHelper.narrow(observerObj));

			log("Observer set. Waiting for events...");

			//orb.run();
			while (true)  {
				try {
					Thread.sleep(500); 																// Tar pause 500 ms
				} catch (InterruptedException interrupted)
					{}
			}


		} catch (Exception e) {
			log("Error: " + e.toString());

		}

	}

	/**
	 * Writes a message to the serial port
	 * @param msg
	 */
	private void log(String msg) {
		SerialPortLogger.getInstance().write(msg);
	}

	protected void pauseApp() {
		// This is not needed
	}

	protected void destroyApp(boolean arg0) throws MIDletStateChangeException {
		// This is not needed
	}

}
