package com.nokia.m2m.sdk.examples.device;

import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import org.omg.CORBA.Object;

import com.nokia.m2m.orb.idl.terminal.ET;
import com.nokia.m2m.orb.idl.terminal.ETHelper;
import com.nokia.m2m.orb.idl.wirelessdevice.Device;
import com.nokia.m2m.orb.idl.wirelessdevice.DeviceHelper;
import com.nokia.m2m.sdk.examples.logger.SerialPortLogger;

public class ObjectNames extends MIDlet {

	/**
	 * This IMlet demonstrates the usage of the Wireless Device CORBA interface.
	 * IMlet calls Device.getObjectNames() method and prints
	 * returned names to the serial port. After object names are resolved
	 * the code gets Embedded Terminal, ET, object reference using
	 * Device.getObjectRef() method. 
	 * 
	 */
	public void startApp() throws MIDletStateChangeException {
		try {
			log("ObjectNames starting");

			org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(null, null);

			// Resolve reference to the Device object
			String url = "corbaloc::127.0.0.1:19740/Device";
			org.omg.CORBA.Object ref = orb.string_to_object(url);
			Device device = DeviceHelper.narrow(ref);

			log("Resolving ObjectNames");			
			String[] nameList = device.getObjectNames();
			String names = "";
			for (int i = 0; i < nameList.length; i++) {
				names += nameList[i];
				if (i != nameList.length - 1) {
					names += ",";
				}
			}
			log("Object names " + names);

			log("Resolving Embedded Terminal");
			Object etRef = device.getObjectRef("ET");
			ET et = ETHelper.narrow(etRef);

			// Ask IMEI from the embedded terminal
			log("IMEI=" + et.IMEI());

		} catch (Exception e) {
			log("Error: " + e.toString());

		}
	}

	private void log(String msg) {
		SerialPortLogger.getInstance().write(msg);
	}

	public void pauseApp() {
	}

	public void destroyApp(boolean arg0) throws MIDletStateChangeException {
	}

}
