package com.nokia.m2m.sdk.examples.device;

import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import org.omg.CORBA.Object;
import org.omg.CORBA.Policy;

import org.omg.PortableServer.IdAssignmentPolicyValue;
import org.omg.PortableServer.IdUniquenessPolicyValue;
import org.omg.PortableServer.LifespanPolicyValue;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;

import com.nokia.m2m.orb.idl.wirelessdevice.Device;
import com.nokia.m2m.orb.idl.wirelessdevice.DeviceHelper;
import com.nokia.m2m.orb.idl.wirelessdevice.ParamObserverHelper;

import com.nokia.m2m.sdk.examples.logger.SerialPortLogger;

/**
 * This IMlet demonstrates the use of the Device CORBA interface.
 * It sets the parameter observer to observe SignalQuality changes.
 * 
 * @see com.nokia.m2m.sdk.examples.device.SignalQualityObserverImpl
 */
public class SignalQualityIMlet extends MIDlet {

	public void startApp() throws MIDletStateChangeException {

		try {
			log("SignalQualityIMlet starting");

			// Initialize the ORB.
			org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(null, null);

			// 
			// Get the RootPOA.
			//
			org.omg.PortableServer.POA rootPoa;
			rootPoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));

			//
			// Create a new POA. Use persistent policies
			// so that servants activated under
			// this POA will always get the same IOR. This ensures 
			// that when the IMlet is restarted
			// after Nokia 12 module reboot, the new reqistration will 
			// overwrite the existing reqistration.
			//
			Policy[] policies =
				new Policy[] {
					rootPoa.create_id_assignment_policy(IdAssignmentPolicyValue.USER_ID),
					rootPoa.create_id_uniqueness_policy(IdUniquenessPolicyValue.UNIQUE_ID),
					rootPoa.create_lifespan_policy(LifespanPolicyValue.PERSISTENT)};

			//
			// Name the new POA as 'ObserverPOA' this name will be part of
			// the listener objects object key.
			//
			POA observerPoa = rootPoa.create_POA("ObserverPOA", rootPoa.the_POAManager(), policies);

			// 
			// Initialize the servant object.
			//
			SignalQualityObserverImpl observerImpl = new SignalQualityObserverImpl();
			observerPoa.activate_object_with_id("SignalQualityObserver".getBytes(), observerImpl);

			//
			// Activate the POAManager. 
			// Activation starts request processing in all POAs that 
			// use this POAManager.
			//
			rootPoa.the_POAManager().activate();

			//
			// The observer is now ready to receive events.
			// Register the listener into the Module ORB.
			//

			// Resolve the Device object.
			String url = "corbaloc::127.0.0.1:19740/Device";
			org.omg.CORBA.Object ref = orb.string_to_object(url);

			Device device = DeviceHelper.narrow(ref);
			// Get CORBA object reference from servant
			Object observerObj = observerPoa.servant_to_reference(observerImpl);

			log("Setting observer");

			// Start to listen to the SignalQuality parameter.
			device.setParamObserver("SignalQuality", ParamObserverHelper.narrow(observerObj));
			
			log("Observer set. Waiting for events");

			orb.run();

		} catch (Exception e) {
			log("Error: " + e.toString());

		}
	}

	/**
	 * Writes a message to the serial port
	 * @param msg
	 */
	private void log(String msg) {
		SerialPortLogger.getInstance().write(msg);
	}

	public void pauseApp() {
	}

	public void destroyApp(boolean arg0) throws MIDletStateChangeException {
	}

}
