package com.nokia.m2m.sdk.examples.gps;

import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import com.nokia.m2m.orb.idl.gps.EAST;
import com.nokia.m2m.orb.idl.gps.GpsData;
import com.nokia.m2m.orb.idl.gps.GpsModule;
import com.nokia.m2m.orb.idl.gps.GpsModuleHelper;
import com.nokia.m2m.orb.idl.gps.NORTH;
import com.nokia.m2m.orb.idl.gps.NoGpsDataAvailableException;
import com.nokia.m2m.sdk.examples.logger.SerialPortLogger;


public class GpsIMlet extends MIDlet {

	/**
	 * Reads GPS data every 30 seconds and prints location information to serial port 3. 
	 * A GPS receivever supporting NMEA protocol must be attached to serial port  
	 * before running this IMlet. Use Nokia 12 GSM Module Configurator to
	 * set one serial port for GPS usage. 
	 * 
	 */
	public void startApp() throws MIDletStateChangeException {
		try {
			log("GpsIMlet starting");

			org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(null, null);

			// Get refrence to the GPS module.
			String url = "corbaloc::127.0.0.1:19740/ORB/OA/IDL:gps/GpsModule:1.0";
			org.omg.CORBA.Object ref = orb.string_to_object(url);
			GpsModule gpsModule = GpsModuleHelper.narrow(ref);

			String gpsInformation;
			String latitude;
			String longitude;
			String aqtime;
			while (true) {
				try {

					GpsData data = gpsModule.getGpsData();

					// Handle GPS data.
					gpsInformation = "Visible satellites " + data.visibleSatellites + "\n";
					gpsInformation += "Altitude" + data.alt + "\n";

					latitude = data.position.lat.degrees + " degrees ";
					latitude += data.position.lat.minutes + " minutes ";
					latitude += data.position.lat.seconds + " seconds ";
					latitude += data.position.lat.milliseconds + " milliseconds ";
					latitude += data.position.lat.type == NORTH.value ? "North" : "South";
					latitude += "\n";

					longitude = data.position.lon.degrees + " degrees ";
					longitude += data.position.lon.minutes + " minutes ";
					longitude += data.position.lon.seconds + " seconds ";
					longitude += data.position.lon.milliseconds + " milliseconds ";
					longitude += data.position.lon.type == EAST.value ? "East" : "West";
					longitude += "\n";
					
					aqtime = "Time="+data.aquisitionTime + "\n";

					SerialPortLogger.getInstance().write(gpsInformation + latitude + longitude + aqtime);

				} catch (NoGpsDataAvailableException noGpsData) {
					// This exception is raised if the GPS device is 
					// disconnected.
					log(noGpsData.toString());
				}

				Thread.sleep(30000);

			}
		} catch (Exception e) {
			log("Error: " + e.toString());

		}
	}

	/**
	 * Write a message to the serial port
	 * @param msg
	 */
	private void log(String msg) {
		SerialPortLogger.getInstance().write(msg);
	}

	protected void pauseApp() {
	}

	protected void destroyApp(boolean arg0) throws MIDletStateChangeException {
	}

}
