package com.nokia.m2m.sdk.examples.hello.impl;

import java.util.Hashtable;

import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;
import com.nokia.m2m.sdk.examples.logger.SerialPortLogger;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

/**
 * Server IMlet for Greeter.
 * 
 * 
 */

public class HelloServerImlet extends MIDlet {

	private ORB orb = null;

	/**
	 * Constructor for HelloServer.
	 */
	public HelloServerImlet() {
		super();
	}

	protected void startApp() throws MIDletStateChangeException {
		try {
			log("HelloServerImlet starting");

			// Set ORB to IIOP mode
			Hashtable props = new Hashtable();
			props.put("com.nokia.m2m.orb.UseM2MGateway", "no");
			orb = ORB.init(null, props);

			// Get the RootPOA
			POA rootPoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));

			// Create policies for a persistent POA. 
			org.omg.CORBA.Policy[] persistentPolicies =
				new org.omg.CORBA.Policy[] {
					rootPoa.create_id_assignment_policy(IdAssignmentPolicyValue.USER_ID),
					rootPoa.create_id_uniqueness_policy(IdUniquenessPolicyValue.UNIQUE_ID),
					rootPoa.create_lifespan_policy(LifespanPolicyValue.PERSISTENT)};

			// Create the server-POA. We retrieve the POAManager from the RootPOA 
			// and use the persistent policies.
			POA serverPOA = rootPoa.create_POA("ServerPOA", rootPoa.the_POAManager(), persistentPolicies);

			// Now we have a POA, create a servant
			GreeterImpl impl = new GreeterImpl();

			// Activate the servant. The objectKey will be ServerPOA/HelloServer
			serverPOA.activate_object_with_id("HelloServer".getBytes(), impl);

			// The last operation, activating the POAManager.
			// It is a very common mistake to forget this! No requests will be
			// processed if the POAManager of the POA hasn't been activated
			rootPoa.the_POAManager().activate();

			log("HelloServerImlet running");

			// ORB keeps running as long as IMlet
		} catch (Exception e) {
			log(e.toString());
		}

	}

	protected void pauseApp() {
	}

	protected void destroyApp(boolean arg0) throws MIDletStateChangeException {
		if (orb != null) {
			orb.destroy();
		}
		log("HelloServerImlet destroyed");
	}

	private void log(String msg) {
		SerialPortLogger.getInstance().write(msg);
	}

}
