package com.nokia.m2m.sdk.examples.http;

import java.io.DataInputStream;
import java.io.OutputStream;

import javax.microedition.io.Connector;
import javax.microedition.io.HttpConnection;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import com.nokia.m2m.sdk.examples.logger.SerialPortLogger;

/**
 * IMlet demonstrates HTTP connection usage.
 * 
 * Remember to configure HTTP settings to the module before running
 * this example.
 * 
 */
public class HTTPIMlet extends MIDlet {

	public void startApp() throws MIDletStateChangeException {

		// Servers address and port
		//final String HOST = "10.35.1.195";
		final String HOST = "example.com";
		final int PORT = 80;

		HttpConnection conn = null;
		DataInputStream in = null;
		OutputStream out = null;

		log("HTTPImlet starting");

		//
		// When asking a http connection from 
		// Connector the used syntax can be
		//
		// "http://ip:port/resource"
		//  or
		// "http://www.example.com/"
		// 

		// get root page 
		String url = "http://" + HOST + ":" + PORT + "/";

		try {

			// Wait to get network service.
			log("Waiting for 30 seconds");
			Thread.sleep(30000);

			log("Creating connection " + url);
			conn = (HttpConnection) Connector.open(url, Connector.READ);
			conn.setRequestMethod(HttpConnection.GET);

			// This step creates connection and sends the HTTP request
			log("Sending request");
			in = conn.openDataInputStream();

			// HTTP server should reply with status code 200 if succeeded
			int status = conn.getResponseCode();
			log("Status= " + status);

			long len = conn.getLength();
			log("Content lenght= " + len);

			StringBuffer buff = new StringBuffer();
			char ch = 0;

			for (int i=0;i<len;i++) {
				ch = (char) in.read();
				buff.append(ch);
			}

			log("Server returned:" + buff.toString());

		} catch (Exception e) {

			log(e.toString());

		} finally {

			// This block is always executed.
			// Release all resources.

			if (conn != null) {
				try {
					conn.close();
				} catch (Exception ignored) {
				}
			}

			if (in != null) {
				try {
					in.close();
				} catch (Exception ignored) {
				}
			}

			if (out != null) {
				try {
					out.close();
				} catch (Exception ignored) {
				}
			}
		}
	}

	private void log(String msg) {
		SerialPortLogger.getInstance().write(msg);
	}

	public void pauseApp() {
	}

	public void destroyApp(boolean arg0) throws MIDletStateChangeException {
	}

}
