package com.nokia.m2m.sdk.examples.http;

import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import org.omg.CORBA.Any;

import com.nokia.m2m.orb.idl.properties.ParametersPackage.HTTPAuthenticationType;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.HTTPBearerInformationGPRS;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.HTTPBearerInformationUnion;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.HTTPBearerType;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.HTTPContextOpenType;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.HTTPInformation;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.HTTPInformationHelper;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.HTTPLoginType;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.HTTPProxyInformation;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.HTTPSessionSecurity;
import com.nokia.m2m.orb.idl.properties.ParametersPackage.HTTPSessionType;
import com.nokia.m2m.orb.idl.wirelessdevice.Device;
import com.nokia.m2m.orb.idl.wirelessdevice.DeviceHelper;
import com.nokia.m2m.sdk.examples.logger.SerialPortLogger;

/**
 * This IMlet demonstrates HTTP connection parameter setup using
 * Properties CORBA interface. Settings are stored in the persistent memory.
 * 
 */
public class HTTPParametersIMlet extends MIDlet {

	public void startApp() throws MIDletStateChangeException {

		try {

			log("Starting HTTPParametersIMlet");

			org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(null, null);

			// Get Device object
			String url = "corbaloc::127.0.0.1:19740/Device";
			org.omg.CORBA.Object devRef = orb.string_to_object(url);
			Device dev = DeviceHelper.narrow(devRef);

			/*
			 * struct HTTPProxyInformation {
			 *   boolean           proxyEnabled;
			 *   unsigned short    primaryProxyPort;
			 *   unsigned short    secondaryProxyPort;
			 *   string            primaryProxyAddress;
			 *   string            secondaryProxyAddress;
			 * };
			 *
			 */

			//
			// Do not use a proxy.
			//
			HTTPProxyInformation httpProxy = new HTTPProxyInformation(false, (short) 0, (short) 0, "", "");

			/*
			 * struct HTTPBearerInformationGPRS {
			 *  HTTPAuthenticationType    authenticationType;
			 *  HTTPContextOpenType       contextOpenType;
			 *  HTTPLoginType             loginType;
			 *  string                    GPRSAccessPointName; 
			 *  string                    GWIPAddress;         
			 *  string                    userName;
			 *  string                    passWord;
			 * };
			 * 
			 */

			HTTPBearerInformationGPRS gprsBearer =
				new HTTPBearerInformationGPRS(
					HTTPAuthenticationType.CHAP,
					HTTPContextOpenType.ON_DEMAND,
					HTTPLoginType.AUTOMATIC,
					"internet",
					"",
					"",
					"");

			/*
			 * union HTTPBearerInformationUnion switch (HTTPBearerType) {
			 *  case HTTP_CSD:    HTTPBearerInformationCSD     bearerInfoCSD;        
			 *  case HTTP_GPRS:   HTTPBearerInformationGPRS    bearerInfoGPRS;
			 * };
			 * 
			 */

			HTTPBearerInformationUnion bearerInformation = new HTTPBearerInformationUnion();
			bearerInformation.bearerInfoGPRS(gprsBearer);

			//
			// When all components are ready, they
			// are set into the HTTPInformation structure.
			//

			/* 
			 * struct HTTPInformation {
			 *   HTTPSessionType               HTTPActiveSessionType;
			 *   HTTPBearerType                HTTPActiveBearerType;
			 *   HTTPSessionSecurity           HTTPSessionSecurity;
			 *   HTTPProxyInformation          HTTPProxyInformation;
			 *   HTTPBearerInformationUnion    HTTPBearerInformationUnion;
			 * };
			 * 
			 */

			HTTPInformation info =
				new HTTPInformation(
					HTTPSessionType.CO,
					HTTPBearerType.HTTP_GPRS,
					HTTPSessionSecurity.UNSECURE,
					httpProxy,
					bearerInformation);

			// Insert the constructed structure into CORBA Any.
			Any any = orb.create_any();
			HTTPInformationHelper.insert(any, info);

			dev.setParam("HTTPInformation", any);

			log("HTTPParameters set");
		} catch (Exception e) {
			log(e.toString());
		}
	}

	private void log(String msg) {
		SerialPortLogger.getInstance().write(msg);
	}

	protected void pauseApp() {
	}

	protected void destroyApp(boolean arg0) throws MIDletStateChangeException {
	}

}
