package com.nokia.m2m.sdk.examples.iomodule;

import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import org.omg.CORBA.SystemException;

import com.nokia.m2m.orb.idl.iomodule.IOControlHelper;
import com.nokia.m2m.orb.idl.iomodule.PinNumberOutOfRange;
import com.nokia.m2m.orb.idl.iomodule.PinValueOutOfRange;
import com.nokia.m2m.sdk.examples.logger.SerialPortLogger;

public class IOModuleDemo extends MIDlet {

	/**
	 * 
	 * This IMlet demonstrates IO module CORBA API
	 * usage. IMlet sets outputpin 7
	 * on and off.
	 *   
	 */
	public void startApp() throws MIDletStateChangeException {

		try {

			log("IOModuleDemo starting");

			org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(null, null);

			// Get a reference to the IO module
			String url = "corbaloc::127.0.0.1:19740/ORB/OA/IDL:IOModule/IOControl:1.0";
			org.omg.CORBA.Object ref = orb.string_to_object(url);
			com.nokia.m2m.orb.idl.iomodule.IOControl iocontrol = IOControlHelper.narrow(ref);

			log("IOModuleDemo running");
			while (true) {
				try {

					// Toggle pin value on and off
					iocontrol.setOutputPinValue((short) 7, 1);
					Thread.sleep(500);
					iocontrol.setOutputPinValue((short) 7, 0);

				} catch (PinValueOutOfRange valueOutOfRange) {
					log(valueOutOfRange.toString());
				} catch (PinNumberOutOfRange numberOutOfRange) {
					log(numberOutOfRange.toString());
				} catch (SystemException se) {

					log(se.toString() + " minor=" + se.minor);
				} catch (InterruptedException ie) {
				}

			}

		} catch (Exception e) {
			log("Error: " + e.toString());

		}
	}

	/**
	 * Write message to serial port
	 * @param msg
	 */
	private void log(String msg) {
		SerialPortLogger.getInstance().write(msg);
	}

	public void pauseApp() {
	}

	public void destroyApp(boolean arg0) throws MIDletStateChangeException {
	}

}
