package com.nokia.m2m.sdk.examples.iomodule;

import java.util.Hashtable;

import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import org.omg.CORBA.ORB;
import org.omg.PortableServer.IdAssignmentPolicyValue;
import org.omg.PortableServer.IdUniquenessPolicyValue;
import org.omg.PortableServer.LifespanPolicyValue;
import org.omg.PortableServer.POAHelper;

import com.nokia.m2m.orb.idl.iomodule.IOControlHelper;
import com.nokia.m2m.orb.idl.iomodule.IOControlObserverHelper;
import com.nokia.m2m.sdk.examples.logger.SerialPortLogger;

public class IOObserverIMlet extends MIDlet {

	/**
	 * 
	 * This IMlet demonstrates IO module CORBA API
	 * usage. Following code sets IO pin observerver 
	 * some input pins
	 *   
	 */
	public void startApp() throws MIDletStateChangeException {

		try {

			log("IOObserverIMlet starting");

			// Set ORB to IIOP mode
			Hashtable props = new Hashtable();
			props.put("com.nokia.m2m.orb.UseM2MGateway", "no");
			org.omg.CORBA.ORB orb = ORB.init(null, props);

			// Get a reference to the IO module
			String url = "corbaloc::127.0.0.1:19740/ORB/OA/IDL:IOModule/IOControl:1.0";
			org.omg.CORBA.Object ref = orb.string_to_object(url);
			com.nokia.m2m.orb.idl.iomodule.IOControl iocontrol = IOControlHelper.narrow(ref);

			//
			// Observer initialization
			//

			// Get the RootPOA
			org.omg.PortableServer.POA rootPoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));

			// Create policies for a persistent POA. 
			org.omg.CORBA.Policy[] persistentPolicies =
				new org.omg.CORBA.Policy[] {
					rootPoa.create_id_assignment_policy(IdAssignmentPolicyValue.USER_ID),
					rootPoa.create_id_uniqueness_policy(IdUniquenessPolicyValue.UNIQUE_ID),
					rootPoa.create_lifespan_policy(LifespanPolicyValue.PERSISTENT)};

			// Create the server-POA. We retrieve the POAManager from the RootPOA 
			// and use the persistent policies.
			org.omg.PortableServer.POA serverPOA =
				rootPoa.create_POA("ServerPOA", rootPoa.the_POAManager(), persistentPolicies);

			// Now we have a POA, create a servant
			IOControlObserverImpl impl = new IOControlObserverImpl();

			// Activate the servant. 
			serverPOA.activate_object_with_id("IOObserver".getBytes(), impl);

			// The last operation, activating the POAManager.
			// It is a very common mistake to forget this! No requests will be
			// processed if the POAManager of the POA hasn't been activated
			rootPoa.the_POAManager().activate();

			log("Setting observer");
			// Set observer to digital input pins 7,8,9
			short[] pinList = new short[] { 7, 8, 9 };
			iocontrol.setIOControlObserver(
				IOControlObserverHelper.narrow(serverPOA.servant_to_reference(impl)),
				pinList);

			log("IOObserverIMlet running");

			// Block execution here
			orb.run();

		} catch (Exception e) {
			log("Error: " + e.toString());

		}
	}

	/**
	 * Write message to serial port
	 * @param msg
	 */
	private void log(String msg) {
		SerialPortLogger.getInstance().write(msg);
	}

	public void pauseApp() {
	}

	public void destroyApp(boolean arg0) throws MIDletStateChangeException {
	}

}
