package com.nokia.m2m.sdk.examples.socket;

import java.io.InputStream;
import java.io.OutputStream;

import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import com.nokia.m2m.sdk.examples.logger.SerialPortLogger;

/**
 * This IMlet demostrates client socket usage. IMlet connects
 * to server using StreamConnection, sends data to server and
 * reads response.
 * 
 */
public class TCPClient extends MIDlet {

	protected void startApp() throws MIDletStateChangeException {
		log("TCPClient starting");

		StreamConnection conn = null;
		OutputStream out = null;
		InputStream in = null;

		try {

			String msg = "HELLO\n";

			// Wait a while for the Nokia 12 module to get into network
			// service after boot-up.
			Thread.sleep(20000);

			log("Connecting...");
// 			conn = (StreamConnection) Connector.open("socket://10.35.1.195:9001");
			conn = (StreamConnection) Connector.open("socket://129.241.219.142:6667");
			log("Connected");

			// Open streams for sending and receiving.
			out = conn.openOutputStream();
			in = conn.openInputStream();

			log("Sendign request");
			// Send a message to the server.
			out.write(msg.getBytes());

			// Read the response.
			log("Reading response");
			byte[] responseBytes = new byte[msg.length()];

			int offset = 0;
			int ret = 0;
			int length = msg.length();

			//
			// The read() call may return less data than was 
			//	enquired.
			//
			while (offset < length) {
				ret = in.read(responseBytes, offset, length - offset);
				if (ret == -1) {
					// Connection is closed by the server.
					log("Error: connection closed");
					return;
				} else {
					offset += ret;
				}
			}

			// Check the response.
			String response = new String(responseBytes);

			if (response.equals(msg)) {
				log("OK! server returned:" + response);
			} else {
				log("FAIL! invalid response:" + response);
			}
		} catch (Exception e) {
			log("Exception: " + e.toString());
		} finally {

			// Write clean up code in the finally block to
			// make sure that all resources are released.

			// Close all open streams and connections.
			if (in != null) {
				try {
					in.close();
				} catch (Exception ignored) {
				}
			}
			if (out != null) {
				try {
					out.close();
				} catch (Exception ignored) {
				}
			}

			// close connection
			if (conn != null) {
				try {
					conn.close();
				} catch (Exception ignored) {
				}
			}

		}

	}

	private void log(String msg) {
		SerialPortLogger.getInstance().write(msg);
	}

	public void pauseApp() {
	}

	public void destroyApp(boolean arg0) throws MIDletStateChangeException {
	}

}
