package com.nokia.m2m.sdk.examples.socket;

import java.io.InputStream;
import java.io.OutputStream;

import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;
import javax.microedition.io.StreamConnectionNotifier;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import com.nokia.m2m.sdk.examples.logger.SerialPortLogger;

/**
 * IMlet demonstrates StreamConnectionNotifier usage
 *
 */
public class TCPServer extends MIDlet {

	public void startApp() throws MIDletStateChangeException {

		StreamConnectionNotifier notifier = null;
		InputStream in = null;
		OutputStream out = null;
		StreamConnection conn = null;

		// Use a buffer to copy data.
		byte[] buffer = new byte[1024];

		try {

			// Creates a server socket and binds to port 9001
			// This method does not open wireless link.
			// If the Nokia 12 Concept Simulator is used, use URL "serversocket://:9001"
			notifier = (StreamConnectionNotifier) Connector.open("socket://:9001");

			while (true) {

				// Sets the socket into the listening mode.
				// The call blocks until
				// a new connection is accepted.

				conn = notifier.acceptAndOpen();

				try {

					in = conn.openInputStream();
					out = conn.openOutputStream();

					//
					// Read data in 1024 byte chunks. Avoid using the 
					// int InputStream.read() reading because it
					// causes a native read for each byte. 
					//

					//
					// The read() call may return
					// less data than was enquired.
					// read() returns the number of bytes received
					//
					int ret = 0;
					while (ret != -1) {

						//
						// Read the data. Returns -1 when the connection is 
						//	closed by the remote peer.
						// The method blocks until
						// something is received.
						//
						ret = in.read(buffer, 0, buffer.length);

						if (ret == -1) {
							// The connection is closed by the server.
							log("connection closed");
						} else {
							out.write(buffer, 0, ret);
						}
					}

				} catch (Exception e) {

					// This exception can be
					// caused by an unexpected link drop. 

					log("Exception while transmitting " + e.toString());
				} finally {

					//
					// Make sure that the streams are closed before
					// going to the next round.  
					//
					if (out != null) {
						try {
							out.close();
						} catch (Exception ignored) {
						}
					}

					if (in != null) {
						try {
							in.close();
						} catch (Exception ignored) {
						}
					}

					if (conn != null) {
						try {
							conn.close();
						} catch (Exception ignored) {
						}
					}

					in = null;
					out = null;
					conn = null;

				} // try

			} // while

		} catch (Exception e) {
			log(e.toString());
		}

	}

	private void log(String msg) {
		// Log to the serial port
		SerialPortLogger.getInstance().write(msg);
	}

	public void pauseApp() {

	}

	public void destroyApp(boolean arg0) throws MIDletStateChangeException {

	}

}
