package com.nokia.m2m.sdk.examples.watchdog;

import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import org.omg.CORBA.ORB;

import com.nokia.m2m.imp.iocontrol.IOControl;
import com.nokia.m2m.imp.watchdog.WatchdogTimer;
import com.nokia.m2m.sdk.examples.logger.Log;
import com.nokia.m2m.sdk.examples.logger.RMSLogger;
import com.nokia.m2m.sdk.examples.logger.SerialPortLogger;

/**
 * <p>
 * This IMlet demonstrates Watchdog timer usage. IMlet
 * reads analog input pin in loop and the Watchdog timer is used
 * to make sure that the IMlet keeps running.
 * </p>
 * 
 * <p>IMlet uses RMSLogger to
 * write messages to record store files. 
 * </p>
 * 
 * @see com.nokia.m2m.sdk.examples.logger
 */
public class WatchdogIMlet extends MIDlet {

	private Log rmsLogger;

	public void startApp() throws MIDletStateChangeException {

		final int pin = 1;
		int value;

		logSerial("WatchdogIMlet starting");

		// Start timer
		WatchdogTimer wd = new WatchdogTimer();
		wd.setTimeout(10 * 60); // 10 minutes

		try {

			// Initialise RMSLogger
			ORB orb = ORB.init(null, null);
			rmsLogger = RMSLogger.getInstance(orb);

			log("WatchdogIMlet started");

			// Read input pin value every 60 seconds
			while (true) {

				value = IOControl.getInstance().getAnalogInputPin(pin);
				log("port " + pin + " value " + value);

				// Reset timer after every successful read
				wd.resetTimer();
				// Wait 60 seconds
				Thread.sleep(60000);

			}

		} catch (Exception ignored) {
			// Code above shoult not throw exceptions
			// Exit loop. WD timer will reset module after 10 minutes
			log(ignored.toString());
		}

	}

	/**
	 * Write log message
	 * @param msg
	 */
	private void log(String msg) {

		// Write also to serial port
		logSerial(msg);
		try {
			logRms(msg);
		} catch (Exception e) {
			logSerial(e.toString());
		}

	}

	/**
	 * Write log to RMS file
	 * @param msg
	 */
	private void logRms(String msg) {
		rmsLogger.write(msg);
	}

	/**
	 * Write log to serial port
	 * @param msg
	 */
	private void logSerial(String msg) {
		SerialPortLogger.getInstance().write(msg);
	}

	public void pauseApp() {
	}

	public void destroyApp(boolean arg0) throws MIDletStateChangeException {
	}

}
