package com.telenor.agps;

import javax.comm.*;
import java.net.*;
import java.io.*;
import java.util.Enumeration;
import java.util.TooManyListenersException;

public class AGPSClient implements SerialPortEventListener {
	// Configuration
	final static String msisdn = "99286868";
	final static String server_name = "reidun.no-ip.com";
	final static int server_port = 6667;
	final static String port_name = "COM1";
	final static int baud_rate = 57600;

	// UBX protocol headers
	final static String AID_DATA  = "#b5#62#0b#10";		// Start of AID_DATA msg
	final static String CFG_MSG   = "0xb50x620x060x010x060x000x0b0x000x000x000x010x000x190xb9";	// Request aiding on target 2
	final static String CFG_RST   = "0xb50x620x060x040x040x000xff0x070x020x000x160x79";	// Cold reset

	boolean running = true;
	boolean debug = true;
	SocketClient client = null;
	Socket socket;
	SerialInterface serial;

	public static void main(String[] args) {
//-------------

	      Enumeration  portList = CommPortIdentifier.getPortIdentifiers();						// Enumeration lager en serie med elementer som kna hentes suksesivt

				System.out.println("Listing all ports");
        while (portList.hasMoreElements()) {
            CommPortIdentifier portIdx = (CommPortIdentifier) portList.nextElement();  			// Henter id'ene til alle portene
            System.out.println("Found " + portIdx.getName()); 									// Henter og skriver ut navnet til alle portene
        }

//-------------
		AGPSClient me = new AGPSClient();														// Lager et nytt objekt av typen AGPSClient og starter klienten
		me.start();
	}
	protected void shutdown() {																	// Metoden shutdown setter parameteren running til false
		running = false;
	}

	public void start() {
		// Dette er metoden som tar seg av oppstarten av klienten

		// Add shutdown hook
		Runtime.getRuntime().addShutdownHook(new Thread() {										// Lager en shutdownhook som alltid kjres ved stopping av klienten
			public void run()
			{
				try {
					if(socket != null)socket.close(); // lagt til test SJJ 2005. Stenger socketen hvis denne fortsatt er igang
					running = false;
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		});

		// Open serial interface and register this class as listener
		serial = new SerialInterface();															// Lager et nytt objekt av typen Serialinterface
		try {
			serial.openSerialPort(port_name, baud_rate, this);									// pner serieporten med navnet port_name, med anngit baudrate
		} catch (Exception e) {
			e.printStackTrace();
		}

		// Cold reset of gps receiver
		serial.sendString(CFG_MSG);																// Foretar en kaldreset av gpsmottakeren
		serial.sendString(CFG_RST);
	}

	public void getAidingFromServer() {
		if (debug) System.out.println("Get aiding data from agps server");

		// Start socketclient if not started
		if (client == null)
			client = new SocketClient();
		if (!client.isrunning)
			client.start();

		client.getAidingData();
	}

	/** Handle serial port events.
	 */
	public void serialEvent(SerialPortEvent event) {
		switch (event.getEventType()) {
			case SerialPortEvent.DATA_AVAILABLE:
				try {
					// Read message from UBlox
					int c;
					String str;
					InputStream m_serialPortReader = serial.m_serialPort.getInputStream();
					String msg = "";
					int count = 0;
					int msgLength = 0;
					boolean readOneMsg = true;

					while (readOneMsg) {
						c = m_serialPortReader.read();	// read 1 byte from serial port
						count++;
						str = Integer.toHexString(c);
						if (str.length() == 1)
							str = "#0" + str;
						else
							str = "#" + str;
						msg += str;		// Add byte to msg

						// Read length of message. Byte 5 and 6 contains length of payload. Msg is payload + 8 bytes
						if (count == 5) msgLength = c;
						if (count == 6) msgLength += c*256;
						if (count == (msgLength+8)) {
							// End of message
							readOneMsg = false;
						}
					}
					if (debug)  System.out.println(msg);
					if (msg.startsWith(AID_DATA)) {
						// GPS receiver requests aiding data
						getAidingFromServer();
					}
				} catch (IOException e) {
					e.printStackTrace();
				}
				break;
		}
	}

	private class SocketClient extends Thread {
		PrintWriter out;
		BufferedReader in;
		boolean isrunning = false;

		public SocketClient() {
			try {
				socket = new Socket(server_name, server_port);
				in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
				out = new PrintWriter(socket.getOutputStream(), true);
				out.println("$LOG connection established");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		public void getAidingData() {
			// Initialize with msisdn and cell_id
			out.println("$INIT,UBX," + msisdn);
			// Ask for INI, ALM and EPH data
			out.println("$INI");
			out.println("$ALM");
			out.println("$EPH");
		}

		public void run() {
			isrunning = true;
			String inputLine = "";
			int c = 0;

			try {
			while (isrunning) {
				// Wait until a line of string data has arrived
				inputLine = in.readLine();

				// Msg can be on several lines
				while (in.ready()) {
					inputLine = inputLine + in.readLine();
				}
				if (inputLine == null) {
					// Connection was closed by server
					shutdown();
				} else {
					if (debug)
						System.out.println("RECV " + inputLine.length() + " bytes: " + (inputLine.length() > 20 ? inputLine.substring(0, 19) : inputLine));
					if (inputLine.startsWith("$INIT_ACK")) {

					} else if (inputLine.startsWith("$ALM")) {
						// Receiving almanac data
						byte[] almData = Base64.decode(inputLine.substring(5));
						for (int i = 0; i < almData.length; i++) {
							c = almData[i] & 0xff;
							serial.sendRaw(c);
							if (debug) {
								System.out.print((c <= 0xf) ? "0x0" : "0x");
								System.out.print(Integer.toHexString(c) + " ");
							}
						}
						if (debug) System.out.println();
						out.println("$ALM_ACK");	// almanac received OK
					} else if (inputLine.startsWith("$EPH")) {
						// Receiving ephemeris data
						byte[] ephData = Base64.decode(inputLine.substring(5));
						for (int i = 0; i < ephData.length; i++) {
							c = ephData[i] & 0xff;
							serial.sendRaw(c);
								if (debug) {
									System.out.print((c <= 0xf) ? "0x0" : "0x");
									System.out.print(Integer.toHexString(c) + " ");
							}
						}
						if (debug) System.out.println();
						out.println("$EPH_ACK");	// ephemeris received OK
					} else if (inputLine.startsWith("$INI")) {
						// Receiving ini data
						byte[] iniData = Base64.decode(inputLine.substring(5));
						for (int i = 0; i < iniData.length; i++) {
							c = iniData[i] & 0xff;
							serial.sendRaw(c);
							if (debug) {
								System.out.print((c <= 0xf) ? "0x0" : "0x");
								System.out.print(Integer.toHexString(c) + " ");
							}
						}
						if (debug) System.out.println();
						out.println("$EPH_ACK");	// ephemeris received OK
					}
				}
			}

			socket.close();
		} catch (SocketException e) {
			System.out.println("Unable to connect to server.");
		} catch (IOException e) {
			e.printStackTrace();
		}

		}
	}
}
