package com.telenor.agps;

import org.apache.log4j.Logger;

import java.util.prefs.Preferences;
import java.util.Vector;
import java.io.FileInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

/**
 * User: Ingebrigt Fuglem, Telenor FoU
 * Date: 25.okt.2004
 * Time: 13:14:32
 */
public class AGPS_Server {

	private static Preferences userPrefs = null;									// Oppretter en node av typen preferences
	private static String VERSION = "0.1.0";
	private DG16Driver driver;

	public AGPS_Server() {
		Logger.getLogger(AGPS_Server.class).info("AGPS server starting up...");		// Lager en logger med navnet AGPS_Server.class.info og legger inn teksten

		// Read config file
		try {
			FileInputStream is = new FileInputStream("config.xml");
			Preferences.importPreferences(is);
			userPrefs = Preferences.userNodeForPackage(this.getClass());
		} catch (Exception e) {
			e.printStackTrace();													// Fanger opp og skriver exceptions
		}

		// Add shutdown hook
		Runtime.getRuntime().addShutdownHook(new Thread() {							// Nr en virtuell maskin starter sin shutdownrutine blir alle shutdownhooks kjrt.
			public void run()
			{
				DBService.getInstance().shutdown();									// Stopper connection mot databasen
				driver.finalize();													// Stopper DG16driveren
				driver.closeSerialPort();											// Stenger serieporten

				Logger.getLogger(AGPS_Server.class).info("Shutdown completed. Goodbye!");
			}
		});

		SocketServer server = new SocketServer();									// Lager et nytt socketserver objekt
		server.start();																// kjrer startmetoden i socketserver

		startDG16();																// kjrer metoden startDG16
		//test();
	}

	public void startDG16() {
		// Start DG16Driver
		String portName = userPrefs.get("SERIAL_PORT", "COM1");						// -henter portnavnet til serieporten fra preference filen til variabelen portnavn
		Logger.getLogger(AGPS_Server.class).debug("Starting DG16Driver Driver on " + portName + "...");
		Logger.getLogger(AGPS_Server.class).info("Press 'CTRL-C' to terminate.");
		// Create the driver
		driver = new DG16Driver();													// Lager et nytt objekt av typen DG16Driver
		try {
			driver.openSerialPort(portName, 9600);									// Kjrer metoden openSerialPort i objektet driver
			driver.initialize();													// Kjrer metoden initialize i objektet driver
		} catch (Exception ex) {
			Logger.getLogger(AGPS_Server.class).error("Unable to connect to GPS receiver");// hvis feil log.
		}

		while (true) {
			// While lkken kjres alltid. Det er her innhenting av data foregr. Innhentingen vil foreg hver time.
			Logger.getLogger(AGPS_Server.class).debug("Get GPS Almanac data from receiver...");
			if (driver.isOK()) {													// Hvis driveren er ok skal data hentes fra GPS mottakeren
				driver.pollAlmanac();
				driver.pollEphemeris();
			}

			// Sov i 1 t
			try {
				Thread.sleep(1*60*60*1000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	public void test() {
		// Dette er en metode som ikke kjres. UBXDriver vil omforme formatet p dataene til UBXformatet som kreves av uBloxen
		UBXDriver ubx = new UBXDriver();

/*
		// getAlmanac
		Vector almanacs = DBService.getInstance().getAlmanac();
		System.out.println("SVID        |   WEEK    |   WORD 0  |   WORD 1  |   WORD 2  |   WORD 3  |   WORD 4  |   WORD 5  |   WORD 6  |   WORD 7  | CHKSUM");
		for (int i = 0; i < almanacs.size(); i++) {
			Almanac almanac = (Almanac) almanacs.elementAt(i);
			String message = ubx.getAID_ALM(almanac);

			int[] bytes = new int[message.length()];
			for (int j=0; j<message.length(); j++) {
					bytes[j] = message.charAt(j) & 0x000000ff;
					String str = Integer.toHexString(bytes[j]);
					if (str.length() == 1) System.out.print("0");
					System.out.print(Integer.toHexString(bytes[j])+"#");
			}
			System.out.println();
		}
*/
/*
		// getEphemeris
		Vector ephemerises = DBService.getInstance().getEphemeris();
		System.out.println("HEADER ID LENGTH |   SVID    |   HOW     |   SF1D0   |   SF1D1   |   SF1D2   |   SF1D3   |   SF1D4   |   SF1D5   |   SF1D6   |   SF1D7   |   SF2D0   |   SF2D1   |   SF2D2   |   SF2D3   |   SF2D4   |   SF2D5   |   SF2D6   |   SF2D7   |   SF3D0   |   SF3D1   |   SF3D2   |   SF3D3   |   SF3D4   |   SF3D5   |   SF3D6   |   SF3D7   | CHKSUM");
		for (int i = 0; i < ephemerises.size(); i++) {
			Ephemeris ephemeris = (Ephemeris) ephemerises.elementAt(i);
			String message = ubx.getAID_EPH(ephemeris);
		}
*/

/*
		// getIni
		System.out.println(" header            00 01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 chksm");
		try {
			String message = ubx.getAID_INI();
		} catch (IOException e) {
			e.printStackTrace();
		}
*/
	}

	public static void main(String[] args) {
		AGPS_Server me = new AGPS_Server();											// Starter serveren
	}
}
