package com.telenor.agps;

import java.util.*;
import java.lang.*;
import java.io.*;
import java.text.NumberFormat;
import java.text.DecimalFormat;
import javax.comm.*;														// En gratis extension API. M lastes ned

import gnu.getopt.*;
import org.apache.log4j.Logger;

/** Driver for a device speaking the National Marine Electronics Association
 * (NMEA) protocol including most Global Positioning System devices.
 *
 *
 * @version
 *
 * @author
 */
public class DG16Driver implements SerialPortEventListener {
	// Definere variabler
	private String lastNMEA = "";
	private long lastNMEAdate = 0;
	private boolean serialOK = false;
	boolean debug = true;
	boolean logtofile = false;
	FileWriter logfile = null;

	/** Create an NMEA device driver.
	 */
	public DG16Driver() {

		// init serial port variables
		m_serialPort = null;
		//m_serialPortReader = null;
		m_serialPortWriter = null;
	}

	/** Open the specified serial port at the given baud rate.
	 */
	public void openSerialPort(String portName, int baudRate) throws NoSuchPortException, PortInUseException, IOException, UnsupportedCommOperationException, TooManyListenersException {
		// close the port if it's open
		closeSerialPort();													// Kjrer metoden close serialport

		// open the select port by name
		CommPortIdentifier portId = CommPortIdentifier.getPortIdentifier(portName);	// Henter portid til en port som alerede er pnet av applikasjonen
		m_serialPort = (SerialPort) portId.open(Integer.toString(hashCode()), 2000);// pner serieporten. Hashcode er en unik ident til et objekt
		if (m_serialPort == null) {													// Hvis man ikke fr pnet porten skriv feilmelding og set variabelen serialOK til false
			System.err.println("Error opening port " + portId.getName());
			serialOK = false;
			return;
		}

		// open the input and output streams
		try {
			//m_serialPortReader = new BufferedReader(new InputStreamReader(m_serialPort.getInputStream()));
			m_serialPortWriter = new BufferedWriter(new OutputStreamWriter(m_serialPort.getOutputStream())); //pner en utgangsstrm for dataene til serieporten
		} catch (IOException e) {													// Fanger exceptions og skriver feilmelding og setter variabelen serialOK til false
			e.printStackTrace();
			System.err.println("Cannot open serial port streams: " + e.toString());
			serialOK = false;
			return;
		}

		// configure the port
		m_serialPort.addEventListener(this);
		m_serialPort.notifyOnDataAvailable(true);
		m_serialPort.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
		m_serialPort.setSerialPortParams(baudRate,
				  SerialPort.DATABITS_8,
				  SerialPort.STOPBITS_1,
				  SerialPort.PARITY_NONE);

		// Skriver ut konfigurasjonen p skjermen
		System.out.println("baud rate: " + m_serialPort.getBaudRate());
		System.out.println("data bits: " + m_serialPort.getDataBits());
		System.out.println("stop bits: " + m_serialPort.getStopBits());
		System.out.println("parity:    " + m_serialPort.getParity());

		serialOK = true;
	}

	/** Close serial port and remove the event listener.
	 */
	public void closeSerialPort() {
		if (m_serialPort != null) {
			m_serialPort.close();
			m_serialPort = null;
		}
		serialOK = false;
	}

	public boolean isOK() {
		return serialOK;
	}

	/**
	 *  Returns the list of all known serial ports.
	 *
	 *@return    List of ports.
	 */
	public static String[] getPortList() {
		// Denne metoden lager en liste over alle tilgjengelige serieporter
		Vector v = new Vector();
		Enumeration portList = CommPortIdentifier.getPortIdentifiers();

		while (portList.hasMoreElements()) {
			CommPortIdentifier portId = (CommPortIdentifier) portList.nextElement();
			//System.out.println(portId.getName());
			if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) {
				v.add(portId.getName());
			}
		}

		return (String[]) v.toArray(new String[v.size()]);								// returnerer en array of strings med navnene til de tilgjengelige serieportene
	}

	/** Handle a serial port events.
	 */
	public void serialEvent(SerialPortEvent event) {
		switch (event.getEventType()) {
			case SerialPortEvent.BI:
			case SerialPortEvent.OE:
			case SerialPortEvent.FE:
			case SerialPortEvent.PE:
			case SerialPortEvent.CD:
			case SerialPortEvent.CTS:
			case SerialPortEvent.DSR:
			case SerialPortEvent.RI:
			case SerialPortEvent.OUTPUT_BUFFER_EMPTY: 									// Hvis utgangsbufferet er tomt skriv en melding om dette p skjermen
				System.out.println("SerialPortEvent: " + event.getEventType() + " (" + event.toString() + ")");
				break;
			case SerialPortEvent.DATA_AVAILABLE:										// Hvis data er tilgjengelig p serieporten
				String inputLine = "";
				try {

					int c;
					boolean cr=false;
					System.out.println("Repeat until '" + Integer.toHexString('\r') + "' + '" + Integer.toHexString('\n') + "'");	// Skriver melding p skjermen
					InputStream m_serialPortReader = m_serialPort.getInputStream();		// oppretter en inngangstrm mot serieporten
					while ((c = m_serialPortReader.read()) != -1) {						// S lenge inngangsstrmmen ikke er lik -1 s utfr whilelkken. Denne legger ogs dataene i inngangsstrmmen i variabelen c
						if (debug) {
							String str = Integer.toHexString(c);
							if (str.length() == 1) System.out.print("0");
							System.out.print(Integer.toHexString(c) + " ");
						}
						inputLine += (char)c;											// Caster inngangsdataene til characters og legger disse inn i Stringvariabelen inputline
						if (c == '\n' && cr) {											// Sjekker om tegnet new line kommer og variabelen cr er satt
							// got <CR><LF>
							if (debug) System.out.println();
							handleInputLine(inputLine);
							inputLine = "";
						} else {
							cr = false;
							if (c == '\r') cr=true;										// Hvis tegnet er carriage return sett variabelen cr
						}
						c=0x00;															// Setter variabelen c til 0
					}
				} catch (IOException e) {
					e.printStackTrace();												// tar seg av exception
				}
				break;
		}
	}

	protected void send(String data) {
		// Dette er en metode som sender data ut p serieporten
		if (!serialOK)																	// Sjekker om serieporten er satt opp riktig
			return;
		try {
			m_serialPortWriter.write(data);												// Sender tekstrengen som kom som inngangsparameter til metoden ut p den pne serieporten
			m_serialPortWriter.newLine();												// Skriver newline tegnene ut p serieporten
			m_serialPortWriter.flush();													// Flusher utgangsbufferet
		} catch (IOException e) {
			e.printStackTrace();														// Tar seg av exceptions
		}

	}

	/** Overridden to make sure the serial port is closed before the object is
	 * garbage collected.
	 */
	public void finalize() {
		// Dette er en metode som stenger frst stopper raporteringen fra GPS modulen DG16 fr den stenger serieporten og stenger logfilen
		send("$PASHS,NME,ALL,A,OFF");													// Sender en streng ut p serieporten for  stoppe rapporteringen fra GPSmodulen
		closeSerialPort();																// Stenger serieporten
		if (logfile != null) {
			try {
				logfile.close();														// Stenger logfilen
			} catch (IOException e) {													// Tar seg av exceptions
				e.printStackTrace();
			}
		}
	}

	protected void initialize() {
		// Dette er en metode som initialiserer DG16 GPSmodulen
		send("$PASHQ,PRT");																// Spr GPSmodulen om serieportenes baudratesettinger
		send("$PASHQ,RID");																// Spr GPSmodulen om GPSmottakerens identifikasjonsparametre

		// Start NMEA signaling
		/*send("$PASHS,NME,SAT,A,ON");
		send("$PASHS,NME,POS,A,ON");*/

		send("$PASHQ,RAW");		// Query raw data settings - not used
		send("$PASHS,RCI,1");	// Set output interval of raw messages to 1 sec

	}

	public void pollAlmanac() {
		send("$PASHQ,SAL");																// Spr GPSmodulen om almanakken til satelittene
	}

	public void pollEphemeris() {
		send("$PASHQ,SNV");																// Spr GPSmodulen om ephemeris til satelittene
	}

	/** Parse and dispatch PASHS sentences.  Corrupt sentences are silently
	 * dropped.
	 */
	protected void handleInputLine(String sentence) {
		// Dette er en metode som tar seg av inngangsdata fra GPSmodulen
		logToFile(sentence);															// Logger inngangsdata
		String payload = sentence;														// Setter strengen payload lik inngangsparameteren
		//if (debug) System.out.println(sentence);
		if (sentence == null || sentence.compareTo("") == 0 || sentence.charAt(0) != '$')// Hvis inngangsparameteren er null, "" eller $ returner
			return;

		// check for start character
		if (sentence.charAt(0) != '$')													// Sjekker om starttegnet i strengen er $. hvis ikke returner
			return;

		//String payload = sentence.substring(1);
/*
		// checksum must verify even though it's optional in the NMEA spec
		int termIndex = sentence.lastIndexOf('*');
		if (termIndex <= 0)
			return;
		String payload = sentence.substring(1, termIndex);
		byte rxChecksum = Byte.parseByte(sentence.substring(termIndex + 1), 16);
		if (rxChecksum != computeChecksum(payload.getBytes())) {
			System.out.println("Checksum failed");
			return;
		}
*/

		// dispatch the sentence
/*
		StringTokenizer st = new StringTokenizer(payload, ",");
		String sentenceName = st.nextToken();
		if (sentenceName.compareTo("PASHR") == 0 && st.hasMoreTokens()) {
			String command = st.nextToken();
			if (command.compareTo("SAL") == 0)
				handleSAL(st.nextToken("\n"));
		}
*/
		String sentenceName = payload.substring(1, 6);								// Henter ut delen av strengen som ligger p plassen 1-5
		if (sentenceName.compareTo("PASHR") == 0 && payload.length() > 11) {		// Sjekker om denne delen er lik PASHR og lengden til den opprinelige strengen er strre enn 11
			String command = payload.substring(7,10);								// Henter ut delen av strengen som ligger p plassen 7-9
			if (command.compareTo("SAL") == 0)										// Sjekker om denne delen er lik SAL, dvs almanakk data
				handleSAL(payload.substring(11));									// Kjrer metoden handleSAL
			if (command.compareTo("SNV") == 0)										// Sjekker om denne delen er lik SNV, dvs ephemeris data
				handleSNV(payload.substring(11));									// Kjrer metoden handleSNV
		}
	}

	protected boolean handleSAL(String raw) {
		// Denne metoden kjres for  ta seg av almanakkdata fra GPSmodulen.
		int[] bytes = new int[raw.length()];										// Lager en array av integere som har samme lengde som inngangsstrenen, dvs nyttelasten fra GPSmottakeren
		for (int i=0; i<raw.length(); i++) {										// Putter inn alle nyttedataene inn i arrayen
				bytes[i] = raw.charAt(i) & 0x000000ff;
		}

/*
		if (debug) {
			Logger.getLogger(DG16Driver.class).debug("Handling SAL message:");
			for (int i=0; i<bytes.length; i++) {
				System.out.print(Integer.toHexString(bytes[i])+"#");
			}
			System.out.println();
		}
*/

		if (bytes.length >= 70) {
			int rxChecksum = binaryToInt(bytes[68], bytes[69]);						// Henter ut checksumen fra nyttedataene
			if (rxChecksum != computeBinaryChecksum(bytes)) {						// Berengner checksumen p nytt og sjekker om disse er like
				Logger.getLogger(DG16Driver.class).error("Error in checksum");		// Checksumene er ulike skriv feilmelding i log
				return false;
			}

			int prn = binaryToInt(bytes[0], bytes[1]);								// Gjr om binrverdien PRN til en int
			int health = binaryToInt(bytes[2], bytes[3]);
			if (health != 0)														// Sjekker om satelittens Health er noe annet enn OK. Returnere hvis ugyldige almanakkdata
				return true;

			double e = binaryToFloat(bytes[4], bytes[5], bytes[6], bytes[7]);
			long toa = binaryToLong(bytes[8], bytes[9], bytes[10], bytes[11]);
			double i0 = binaryToFloat(bytes[12], bytes[13], bytes[14], bytes[15]);
			double omegadot = binaryToFloat(bytes[16], bytes[17], bytes[18], bytes[19]);
			double roota = binaryToDouble(bytes[20], bytes[21], bytes[22], bytes[23], bytes[24], bytes[25], bytes[26], bytes[27]);
			double omega0 = binaryToDouble(bytes[28], bytes[29], bytes[30], bytes[31], bytes[32], bytes[33], bytes[34], bytes[35]);
			double omega = binaryToDouble(bytes[36], bytes[37], bytes[38], bytes[39], bytes[40], bytes[41], bytes[42], bytes[43]);
			double m0 = binaryToDouble(bytes[44], bytes[45], bytes[46], bytes[47], bytes[48], bytes[49], bytes[50], bytes[51]);
			float af0 = binaryToFloat(bytes[52], bytes[53], bytes[54], bytes[55]);
			float af1 = binaryToFloat(bytes[56], bytes[57], bytes[58], bytes[59]);
			int wna = binaryToInt(bytes[60], bytes[61]);
			int wn = binaryToInt(bytes[62], bytes[63]);
			long tow = binaryToLong(bytes[64], bytes[65], bytes[66], bytes[67]);

			Almanac almanac = new Almanac();										// Lager et nytt objekt av typen almanac og legger alle almanakkdataene inn i denne.
			almanac.prn = prn;
			almanac.health = health;
			almanac.e = e;
			almanac.toa = toa;
			almanac.i0 = i0;
			almanac.omegadot = omegadot;
			almanac.roota = roota;
			almanac.omega0 = omega0;
			almanac.omega = omega;
			almanac.m0 = m0;
			almanac.af0 = af0;
			almanac.af1 = af1;
			almanac.wna = wna;
			almanac.wn = wn;
			almanac.tow = tow;

			DBService.getInstance().setAlmanac(almanac);							// Legger dataene inn i databasen
			Logger.getLogger(DG16Driver.class).debug("Got almanac data for satellite number " + (prn+1)); // logger at alamanakk dataene for satelitt er hentet
			if (debug) {
				// Hvis vi debuger vil alamenakkdataene bli skrevet p skjermen
				DecimalFormat nf = new DecimalFormat("0.000000000E000");
				DecimalFormat nf_int = new DecimalFormat("000");
				System.out.println("**** Week " + wn + " almanac for PRN-" + (prn+1) + " **********");
				System.out.println("ID:                             " + nf_int.format((prn+1)));
				System.out.println("Health:                         " + nf_int.format(health));
				System.out.println("Eccentricity:                   " + nf.format(e));
				System.out.println("Time of Applicability(s):       " + nf.format(toa));
				System.out.println("Orbital Inclination(rad):       " + nf.format(i0*Math.PI));
				System.out.println("Rate of Right Ascen(r/s):       " + nf.format(omegadot*Math.PI));
				System.out.println("SQRT(A) (m^1/2):                " + nf.format(roota));
				System.out.println("Right Ascen at TOA(rad):        " + nf.format(omega0*Math.PI));
				System.out.println("Argument of Perigee(rad):       " + nf.format(omega*Math.PI));
				System.out.println("Mean Anom(rad):                 " + nf.format(m0*Math.PI));
				System.out.println("Af0(s):                         " + nf.format(af0));
				System.out.println("Af1(s/s):                       " + nf.format(af1));
				System.out.println("Week:                           " + wn);
				System.out.println();
			}
		} else {
			Logger.getLogger(DG16Driver.class).error("Error in message. Length = " + bytes.length);	// Hvis nyttelasten har ugyldig lengde (dvs kortere enn 70 tegn) skriv feilmelding i logg.
		}
		return true;
	}

	/**
	 * Handle Satellite Ephemeris Data
	 * @param raw
	 * @return
	 */

	protected boolean handleSNV(String raw) {
		// Denne metoden kjres for  ta seg av ephemerisdata fra GPSmodulen.
		int[] bytes = new int[raw.length()];										// Lager en array av bytes og legger dataene fra inngangsstrengen inn i denne
		for (int i=0; i<raw.length(); i++) {
				bytes[i] = raw.charAt(i) & 0x000000ff;
		}

		if (debug) {																// Hvis debugging skriv ut melding p skjermen
			System.out.println("Handling SNV message:");
			for (int i=0; i<bytes.length; i++) {
				System.out.print(Integer.toHexString(bytes[i])+";");
			}
			System.out.println();
		}

		if (bytes.length >= 132) {
			int rxChecksum = binaryToInt(bytes[130], bytes[131]);					// Sjekker checksumen til dataene hvis feil i checksum skriv ut feilmelding og returner
			if (rxChecksum != computeBinaryChecksum(bytes)) {
				System.out.println("Error in checksum");
				return false;
			}

			Ephemeris eph = new Ephemeris();										// lager et nytt objekt av typen Aphemeris og skriver verdiene som er hentet inn i denne
			eph.wn = binaryToInt(bytes[0], bytes[1]);								// Veriene blir konvertert til riktig talltype fr de blir lagret
			eph.tow = binaryToLong(bytes[2], bytes[3], bytes[4], bytes[5]);
			eph.tgd = binaryToFloat(bytes[6], bytes[7], bytes[8], bytes[9]);
			eph.aodc = binaryToLong(bytes[10], bytes[11], bytes[12], bytes[13]);
			eph.toc = binaryToLong(bytes[14], bytes[15], bytes[16], bytes[17]);
			eph.af2 = binaryToFloat(bytes[18], bytes[19], bytes[20], bytes[21]);
			eph.af1 = binaryToFloat(bytes[22], bytes[23], bytes[24], bytes[25]);
			eph.af0 = binaryToFloat(bytes[26], bytes[27], bytes[28], bytes[29]);
			eph.aode = binaryToLong(bytes[30], bytes[31], bytes[32], bytes[33]);
			eph.deltan = binaryToFloat(bytes[34], bytes[35], bytes[36], bytes[37]);
			eph.m0 = binaryToDouble(bytes[38], bytes[39], bytes[40], bytes[41], bytes[42], bytes[43], bytes[44], bytes[45]);
			eph.e =  binaryToDouble(bytes[46], bytes[47], bytes[48], bytes[49], bytes[50], bytes[51], bytes[52], bytes[53]);
			eph.roota = binaryToDouble(bytes[54], bytes[55], bytes[56], bytes[57], bytes[58], bytes[59], bytes[60], bytes[61]);
			eph.toe = binaryToLong(bytes[62], bytes[63], bytes[64], bytes[65]);
			eph.cic = binaryToFloat(bytes[66], bytes[67], bytes[68], bytes[69]);
			eph.crc = binaryToFloat(bytes[70], bytes[71], bytes[72], bytes[73]);
			eph.cis = binaryToFloat(bytes[74], bytes[75], bytes[76], bytes[77]);
			eph.crs = binaryToFloat(bytes[78], bytes[79], bytes[80], bytes[81]);
			eph.cuc = binaryToFloat(bytes[82], bytes[83], bytes[84], bytes[85]);
			eph.cus = binaryToFloat(bytes[86], bytes[87], bytes[88], bytes[89]);
			eph.omega0 = binaryToDouble(bytes[90], bytes[91], bytes[92], bytes[93], bytes[94], bytes[95], bytes[96], bytes[97]);
			eph.omega = binaryToDouble(bytes[98], bytes[99], bytes[100], bytes[101], bytes[102], bytes[103], bytes[104], bytes[105]);
			eph.i0 = binaryToDouble(bytes[106], bytes[107], bytes[108], bytes[109], bytes[110], bytes[111], bytes[112], bytes[113]);
			eph.omegadot = binaryToFloat(bytes[114], bytes[115], bytes[116], bytes[117]);
			eph.idot = binaryToFloat(bytes[118], bytes[119], bytes[120], bytes[121]);
			eph.accuracy = binaryToInt(bytes[122], bytes[123]);
			eph.health = binaryToInt(bytes[124], bytes[125]);
			eph.fit = binaryToInt(bytes[126], bytes[127]);
			eph.prn = bytes[128];

			DBService.getInstance().setEphemeris(eph);								// Lagrer dataene i databasen
			Logger.getLogger(DG16Driver.class).debug("Got ephemeris data for satellite number " + (eph.prn+1)); // Skriver til logg at operasjoen er utfrt

		} else {
			System.out.println("Error in message. Length = " + bytes.length);		// Hvis nyttelasten har feil lengde s skrives feilmelding i loggen
		}
		return true;
	}

	/** Comput a checksum for the given array of bytes.
	 */
	public static byte computeChecksum(byte[] bytes) {
		// Dette er en metode som beregner checksumen Denne brukes ikke lenger
		for (int i = 0; i < bytes.length; i++)
			result = result ^ bytes[i];
		return (byte) result;
	}

	/** Comput a checksum for the given array of bytes.
	 */
	public static int computeBinaryChecksum(int[] bytes) {
		// Dette er en metode som beregner checksumen til nyttelasten. Dette er den samme som den som str i DG16manualen p side 384
		int result = 0;
		int j=0;
		for (int i = 0; i < bytes.length-4; i=i+2) {
			result = result + ((0xff & bytes[i+1]) | ((0xff & bytes[i]) <<8));
		}
		return result & 0xFFFF;
	}

/*
	double binaryToDouble(long bits) {
		int s = ((bits >> 63) == 0) ? 1 : -1;
		int e = (int)((bits >> 52) & 0x0003ffL);
		long m = (e == 0) ?
							 (bits & 0xfffffffffffffL) << 1 :
							 (bits & 0xfffffffffffffL) | 0x10000000000000L;
		System.out.println("s=" + s + ", e=" + e + ", m=" + m);
		double retval = s*m*Math.pow(2, e-1023);
		return retval;
	}
*/

	/**
	 * Convert a 4 byte binary value to a float
	 * @param bytes a 4 bytes binary representation of a float according to IEEE
	 * @return a float as a double
	 */
	float binaryToFloat(int byte0, int byte1, int byte2, int byte3) {
/*
		for (int i=0;i<4;i++)
			System.out.print(Integer.toHexString(bytes[i]));
		System.out.println();
*/
		if (((byte0&0xFF) == 0) & ((byte1&0xFF) == 0) & ((byte2&0xFF) == 0) & ((byte3&0xFF) == 0)) // Sjekker om tallet er null
			return 0;
		if (((byte0&0xFF) == 1) & ((byte1&0xFF) == 1) & ((byte2&0xFF) == 1) & ((byte3&0xFF) == 1)) // Sjekker om tallet er ugyldig. hvis ugyldig returner not a number
			return Float.NaN;

		int[] bytes = {byte0, byte1, byte2, byte3};									// Lager en array of integers og putter  byte-verdiene inn i denne
		// Sign is the first bit
		int sign = ((bytes[0] >> 7) == 0)? 1 : -1;									// Sjekker fortegnet ved  skifte bitene i den frste byten 7 plasser. Sjekker det gjennvrende bitet. hvis 0 returner 1 i variabelen sign, hvis 1 returner -1
		// Exponent is the next 8 bits
		int exp = (((bytes[0] << 1) & 0xFF) + ((bytes[1] >> 7) & 0xFF) - 127);		// Sjekker eksponenten. Denne ligger i de neste 8 bitene. Grunnen til at man trekker ifra 127 er at eksponenten i et binrt tall alltid er anngitt med en bias p 127 for  takle negative eksponenter.

		// Read fraction from the last 23 bits of the four bytes
		double fraction = 1;														// initialiserer variablene. Fraction er verdiene som skal finnes, x er desimalverdiene til bitplassen som behandles n, f.eks 2^-1, b er byteteller
		double x = 1;
		int b = 0;
		for (int i=1; i<24; i++) {
			x = x/2;																// finner desimalverdien til biten som behandles
			if (i % 8 == 0) b++;													// Sjekker om vi har flyttet oss en byte. hvis vi har gjrot det inkrementer bytetelleren
			int bit = (bytes[b+1] >> (7-(i%8))) & 0x0001;							// Setter binrverdien i den biten som sjekkes til variabelen bit
			if (bit == 1) fraction += x;											// hvis bitet var 1 legg til x-verdien i variablen fraction
		}
		double df = sign  * Math.pow(2, exp) * fraction;							// beregner tallet ved  multiplisere fortegn, exponent og fraction
		return (float)df;															// returnere floatverdien (caster double til float
	}
	/**
	 * Convert a 8 byte binary value to a double
	 * @param bytes a 4 bytes binary representation of a float according to IEEE
	 * @return a double
	 */
	double binaryToDouble(int byte0, int byte1, int byte2, int byte3, int byte4, int byte5, int byte6, int byte7) {
		if (((byte0&0xFF) == 0) & ((byte1&0xFF) == 0) & ((byte2&0xFF) == 0) & ((byte3&0xFF) == 0) &
				  ((byte4&0xFF) == 0) & ((byte5&0xFF) == 0) & ((byte6&0xFF) == 0) & ((byte7&0xFF) == 0))  // Hvis tallet er null returner 0
			return 0;
		if (((byte0&0xFF) == 1) & ((byte1&0xFF) == 1) & ((byte2&0xFF) == 1) & ((byte3&0xFF) == 1) &
				  ((byte4&0xFF) == 1) & ((byte5&0xFF) == 1) & ((byte6&0xFF) == 1) & ((byte7&0xFF) == 1))  // Hvis taller er ugyldig returner not a number
			return Double.NaN;

		int[] bytes = {byte0, byte1, byte2, byte3, byte4, byte5, byte6, byte7};		// Lager en array of integers og putter byteverdiene inn i denne.
/*
		for (int i=0;i<8;i++)
			System.out.print(Integer.toHexString(bytes[i]));
		System.out.println();
*/

		// Sign is the first bit
		int sign = ((bytes[0] >> 7) == 0)? 1 : -1;									// Sjekker fortegnet ved  skifte bitene i den frste byten 7 plasser. Sjekker det gjennvrende bitet. hvis 0 returner 1 i variabelen sign, hvis 1 returner -1
		// Exponent is the next 8 bits? er det ikke 11?
		int exp = (((bytes[0] << 4) & 0x7FF) + ((bytes[1] >> 4) & 0x0F) - 1023);	// Sjekker eksponenten. Denne ligger i de neste 11 bitene. Shifter frste byte 4 plasser mot venstre og ander med 7FF for  fjerne det frste bitet(fortegn)
																					// Legger dette sammen med den andre bytene shiftet 4 plasser mot hyre og andet med 0f for  fjerne fractionverdien i dette bitet. Grunnen til at man trekker
																					// ifra 1023 er at eksponenten i et double alltid er anngitt med en bias p 1023 for  takle negative eksponenter.

		// Read fraction from the last 52 bits of the eight bytes
		double fraction = 1;														// initialiserer variablene. Fraction er verdiene som skal finnes, x er desimalverdiene til bitplassen som behandles n, f.eks 2^-1, b er byteteller
		double x = 1;
		int b = 0;
		for (int i=4; i<56; i++) {
			x = x/2;																// finner desimalverdien til biten som behandles
			if (i % 8 == 0) b++;													// Sjekker om vi har flyttet oss en byte. hvis vi har gjrot det inkrementer bytetelleren
			int bit = (bytes[b+1] >> (7-(i%8))) & 0x0001;							// Setter binrverdien i den biten som sjekkes til variabelen bit
			if (bit == 1) fraction += x;											// hvis bitet var 1 legg til x-verdien i variablen fraction
		}
		if ((exp == 0) && (fraction == 0))
			return 0;
		double df = sign  * Math.pow(2, exp) * fraction;							// beregner tallet ved  multiplisere fortegn, exponent og fraction
		//System.out.println("sign="+sign+", exp="+exp+", fration="+fraction);
		return df;																	// returnere doubleverdien
	}

	long binaryToLong(int byte0, int byte1, int byte2, int byte3) {
		return (0x0ff & byte3) | ((0x0ff & byte2) << 8) | ((0x0ff & byte1) << 16) | ((0x0ff & byte0) << 24);	// Setter bare bytene etter hverandre for  beregne long
	}

	int binaryToInt(int byte0, int byte1) {
		return (0x0ff & byte1) | ((0x0ff & byte0) << 8);							// Setter bare bytene etter hverandre for  beregne Integer
	}

	/** Show the program argument usage and then exit.
	 */
	protected static void printUsageAndExit() {
		// Denne metoden skriver ut feilmelding p skjermen og  avslutter hvis DGDriver blir startet p feil mte
		System.err.println("usage: DG16Driver <serial_port>");
		System.exit(1);
	}

	int x=0;
	protected void logToFile(String log) {
		// Denne metoden tar seg av loggingen.
		if (logtofile) {
			try {
				if (logfile == null) {
					logfile = new FileWriter("gps.log");							// oppretter en logfil med navnet gps.log hvis det ikke finnes noen fra fr
				}
				x++;
				logfile.write(x + ": " + log+'\n');									// Skriver linje nummer og logen til filen og avslitter med linjeskift
			} catch (IOException e) {												// Tar seg av exceptions
				e.printStackTrace();
			}
		}
	}

	public static void main(String argv[]) {
		// Parse command line options
		int c;
		// - allows return in order functionality
		Getopt g = new Getopt("DG16Driver", argv, ":");
		g.setOpterr(true); // do own error handling
		while ((c = g.getopt()) != -1) {
			switch (c) {
				case ':':
					System.err.println("An argument is required for option " + (char) g.getOptopt());
					printUsageAndExit();
					break;
				case '?':
					System.err.println("The option '" + (char) g.getOptopt() + "' is not valid");
					printUsageAndExit();
					break;
			}
		}

		// verify that serial port was specified
		if (g.getOptind() >= argv.length)
			printUsageAndExit();
		String portName = argv[g.getOptind()];

		try {
			// Create the driver
			System.out.println("Starting DG16Driver Driver on " + portName + "...");
			DG16Driver driver = new DG16Driver();										// Lager et objekt av typen DG16Driver

			// main operation
			driver.openSerialPort(portName, 9600);										// pner serieporten som er angitt ved kjring av filen
			driver.initialize();														// Kjrer initialize metoden
			BufferedReader input = new BufferedReader(new InputStreamReader(System.in));// Lager en ny buffered reader
			System.out.println("Type 'quit' to terminate.");							// Skriver melding til skjermen

			// loop dispatching command strings until we get 'quit'
			while (true) {
				String cmd = input.readLine();											// leser inn fra tastaturet
				if (cmd.compareToIgnoreCase("quit") == 0)								// Hopper ut av whilelkken hvis quit er skrevet p tastasturet
					break;
			}
			driver.finalize();															// kjrer finalize metoden
			driver.closeSerialPort();													// stenger serieporten

		} catch (Exception ex) {														// Tar seg av exception
			ex.printStackTrace();
		}

		// the exit shuts down rain, otherwise the serive framework continues
		System.exit(0);																	//
	}

	protected SerialPort m_serialPort;
	//protected BufferedReader m_serialPortReader;
	protected BufferedWriter m_serialPortWriter;

	public String getLastNMEA() {
		return lastNMEA;
	}

	public long getLastNMEAdate() {
		return lastNMEAdate;
	}
}
