package com.telenor.agps;

import javax.comm.*;
import java.io.*;
import java.util.*;

/**
 * User: Ingebrigt Fuglem, Telenor FoU
 * Date: 07.des.2004
 * Time: 14:55:18
 */
public class SerialInterface {
	protected SerialPort m_serialPort;
	protected OutputStream out;
	private boolean serialOK = false;

	public SerialInterface() {
		// init serial port variables
		m_serialPort = null;
		out = null;
	}

	/** Open the specified serial port at the given baud rate.
	 */
	public void openSerialPort(String portName, int baudRate, SerialPortEventListener listener) throws NoSuchPortException, PortInUseException, IOException, UnsupportedCommOperationException, TooManyListenersException {
		// close the port if it's open
		closeSerialPort();

		System.out.println("Attempt to open: " + portName);
		// open the select port by name
		CommPortIdentifier portId = CommPortIdentifier.getPortIdentifier(portName);
			
		m_serialPort = (SerialPort) portId.open(Integer.toString(hashCode()), 2000);
		if (m_serialPort == null) {
			System.err.println("Error opening port " + portId.getName());
			serialOK = false;
			return;
		}
		out = m_serialPort.getOutputStream();

		// configure the port
		m_serialPort.addEventListener(listener);
		m_serialPort.notifyOnDataAvailable(true);
		m_serialPort.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
		m_serialPort.setSerialPortParams(baudRate,
				  SerialPort.DATABITS_8,
				  SerialPort.STOPBITS_1,
				  SerialPort.PARITY_NONE);

		System.out.println("baud rate: " + m_serialPort.getBaudRate());
		System.out.println("data bits: " + m_serialPort.getDataBits());
		System.out.println("stop bits: " + m_serialPort.getStopBits());
		System.out.println("parity:    " + m_serialPort.getParity());

		serialOK = true;
	}

	/** Close serial port and remove the event listener.
	 */
	public void closeSerialPort() {
		if (m_serialPort != null) {
			m_serialPort.close();
			m_serialPort = null;
		}
		serialOK = false;
	}

	public boolean isOK() {
		return serialOK;
	}


	public void sendRaw(int data) {
		if (isOK()) {
			try {
				out.write(data);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * Write a string of data to serial interface
	 * Data must be on the form 0x0b0xff
	 * @param data
	 */
	public void sendString(String data) {
		if (isOK()) {
			int c = 0;
			String b="";
			if (data.length()%4 != 0) {
				System.out.println("Error: each byte in data must have 4 characters (0x12). Data=" + data);
				return;
			}
			String str="";
			System.out.println("Sending: " + data);
			for (int i = 0; i < data.length(); i+=4) {
				b=data.substring(i, i+4);
				c = Integer.decode(b).intValue();

				try {
					out.write(c);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}
}
