package com.telenor.agps;

import org.apache.log4j.Logger;

import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.util.prefs.Preferences;
import java.util.*;
import java.io.*;
import java.lang.Byte;
/**
 * User: Ingebrigt Fuglem, Telenor FoU
 * Date: 25.okt.2004
 * Time: 14:21:21
 */

public class SocketServer extends Thread {
	ServerSocket server;
	private static Logger logger = Logger.getLogger(SocketServer.class);
	private final Preferences prefs;
	private static boolean running = true;
	private static Hashtable agpsClients;

	protected SocketServer() {
		agpsClients = new Hashtable();
		prefs = Preferences.userNodeForPackage(this.getClass());
	}

	protected void shutdown() {
		try {
			server.close();
		} catch (IOException e) {
		}
	}

	public void closeConnection(SessionData client) {
		logger.debug("Closing connection with id = " + client.msisdn);
		try {
			client.socket.close();
		} catch (IOException e) {
			Logger.getLogger(this.getClass()).error("", e);
		}
	}

	private String getAlmanac() throws IOException {
		UBXDriver ubx = new UBXDriver();
		Vector almanacs = DBService.getInstance().getAlmanac();
		ByteArrayOutputStream almanacBuffer = new ByteArrayOutputStream();
		for (int i = 0; i < almanacs.size(); i++) {
			Almanac almanac = (Almanac) almanacs.elementAt(i);
			byte[] almData = ubx.getAID_ALM(almanac);
			almanacBuffer.write(almData);
			almanacBuffer.write(13);	//EOL
		}
		return Base64.encodeBytes(almanacBuffer.toByteArray());
	}

	private String getEphemeries() throws IOException {
		UBXDriver ubx = new UBXDriver();
		Vector ephemerises = DBService.getInstance().getEphemeris();
		ByteArrayOutputStream ephemerisBuffer = new ByteArrayOutputStream();
		for (int i = 0; i < ephemerises.size(); i++) {
			Ephemeris ephemeris = (Ephemeris) ephemerises.elementAt(i);
			byte[] ephData = ubx.getAID_EPH(ephemeris);
			ephemerisBuffer.write(ephData);
		}
		return Base64.encodeBytes(ephemerisBuffer.toByteArray());
	}

	private String getIniData() throws IOException {
		UBXDriver ubx = new UBXDriver();
		byte[] iniData = ubx.getAID_INI();
		return new String (iniData);
	}


	public void run() {
		long timeout = prefs.getInt("TIMEOUT_MIN", 1) * 60 * 1000;
		// Start server for incomming connections
		ConnectionHandler ch = new ConnectionHandler();
		ch.start();

		while (running) {
			Enumeration enum;
			synchronized (agpsClients) {
				enum = agpsClients.elements();
			}
			// For each client
			// Check for incomming data
			while (enum.hasMoreElements() && running) {
				SessionData client = (SessionData) enum.nextElement();

				try {
					// Read data from client
					InputStreamReader in = new InputStreamReader(client.socket.getInputStream());
// SJJ					System.out.println("Attempt reading from" + client.socket);
					if (in.ready()) {
						BufferedReader reader = new BufferedReader(in);
						PrintWriter out = new PrintWriter(client.socket.getOutputStream(), true);
						String inputLine = "";
//						while ((inputLine = reader.readLine()) != null) {

						while (reader.ready()) {
							//inputLine = inputLine + reader.readLine();
							inputLine = reader.readLine();
//						}

						client.lastUsed = new Date();
						Logger.getLogger(SocketServer.class).debug("RECV: " + (inputLine.length() > 20 ? inputLine.substring(0, 19) : inputLine));

//	SJJ					System.out.println("Input lenght is: " + inputLine.length());

						// Handle the data received from client
						if ((inputLine.length() > 0) && (inputLine.charAt(0) != '$')) {		// added check on inputLine lenght
							Logger.getLogger(SocketServer.class).warn("Error in command from client. '" + inputLine + "'");
							out.println("$NACK");
						} else if (inputLine.startsWith("$ALM_ACK")) {
							Logger.getLogger(SocketServer.class).debug("Client received almanac data OK");
						} else if (inputLine.startsWith("$ALM")) {
							String almanac = getAlmanac();
							Logger.getLogger(SocketServer.class).debug("Sending " + almanac.length() + " bytes of almanac data");
							out.println("$ALM," + almanac);
						} else if (inputLine.startsWith("$EPH_ACK")) {
							Logger.getLogger(SocketServer.class).debug("Client received ephemeris data OK"); // corrected text
						} else if (inputLine.startsWith("$EPH")) {
							String ephemeries = getEphemeries();
							Logger.getLogger(SocketServer.class).debug("Sending " + ephemeries.length() + " bytes of ephemeris data");
							out.println("$EPH," + ephemeries);
						} else if (inputLine.startsWith("$INIT") && inputLine.length() > 6) {
							StringTokenizer st = new StringTokenizer(inputLine.substring(6), ",");
							client.protocol = st.nextToken();
							client.msisdn = st.nextToken();
							if (st.hasMoreTokens())
								client.cellId = st.nextToken();
							out.println("$INIT_ACK");
						} else if (inputLine.startsWith("$INI")) {
							String iniData = getIniData();
							Logger.getLogger(SocketServer.class).debug("Sending " + iniData.length() + " bytes of ini data");
							out.println("$INI," + iniData);
						} else if (inputLine.startsWith("$LOG")) {
							Logger.getLogger(SocketServer.class).debug("From client: " + inputLine);
							out.println("From client: " + inputLine);
						} else {
							// Unknown command
							Logger.getLogger(SocketServer.class).warn("Error in command from client. '" + inputLine + "'");
							out.println("$NACK");
						}
						}
					} else {
						// Check if client should time-out
						long waitTime = System.currentTimeMillis() - client.lastUsed.getTime();
						if (waitTime > timeout) {
							logger.info("Client with id = " + client.msisdn + " has timed out!");
							closeConnection(client);
							String clientId = client.socket.getInetAddress() + ":" + client.socket.getPort();
							synchronized (agpsClients) {
								agpsClients.remove(clientId);
							}
						}
					}
				} catch (IOException e) {
					Logger.getLogger(SocketServer.class).error("Unknown io-error.", e);
				}
			}

			// Sleep for 100ms and then check next client
			try {
				Thread.sleep(100);
				Thread.yield();
			} catch (InterruptedException e) {
				logger.error("", e);
			}
		}
	}

	/**
	 * Inner class ConnectionHandler
	 * Starts the socketserver and waits for new connections to arrive
	 * New agpsClients are stored in the vector agpsagpsClients
	 */
	private class ConnectionHandler extends Thread {
		ServerSocket server;

		public ConnectionHandler() {
			int port = prefs.getInt("SERVER_PORT", 53);
			try {
				server = new ServerSocket(port);
				logger.info("AGPS Server started on port " + port);
				logger.debug("Socket buffersize: " + server.getReceiveBufferSize());

			} catch (IOException e) {
				logger.error("Unable to create server on port " + port, e);
			}
		}

		protected void shutdown() {
			try {
				server.close();
			} catch (IOException e) {
			}
		}

		public void run() {
			while (running) {
				// Accept incomming connections forever
				try {
					Socket clientConnect = server.accept();
					clientConnect.setSoTimeout(200);
					SessionData client = new SessionData();
					client.socket = clientConnect;
					client.lastUsed = new Date();
					client.msisdn = "";
					client.cellId = "";
					String clientId = client.socket.getInetAddress() + ":" + client.socket.getPort();
					synchronized (agpsClients) {
						agpsClients.put(clientId, client);
					}
					logger.debug("Accepted incoming connection from " + clientId);
				} catch (SocketTimeoutException e) {
				} catch (SocketException e) {
					if (running)
						logger.error("Could not accept connections.", e);
				} catch (IOException e) {
					logger.error("Unknown io-error.", e);
				}
			}
		}
	}

	private class SessionData {
		String msisdn;
		String cellId;
		Socket socket;
		Date lastUsed;
		String protocol;
	}

}

