package com.telenor.agps;

import java.io.StringWriter;
import java.io.IOException;
import java.io.ByteArrayOutputStream;

/**
 * User: Ingebrigt Fuglem, Telenor FoU
 * Date: 25.okt.2004
 * Time: 14:33:01
 */
public class UBXDriver {

	boolean DEBUG = true;

	public byte[] getAID_ALM(Almanac almanac) {
		// Denne metoden skriver ut almanakkdataene
		ByteArrayOutputStream buffer = new ByteArrayOutputStream(44);		// lager et nytt objekt av typen ByteArrayOutputStream

		// ID
		buffer.write(0x0b);													// skriver verdiene 0x0b30 inn i butearrayoutputstreamen
		buffer.write(0x30);

		// Length
		buffer.write(40);													// Skriver inn lengden til nyttelasten inn i strmmen
		buffer.write(0);

		// SVID
		int svid = almanac.prn+1;											// Skriver inn svid inn i strmmen
		buffer.write(svid);
		buffer.write(0);
		buffer.write(0);
		buffer.write(0);

		// WEEK
		buffer.write(almanac.wn & 0xff);									// SKriver ukenummer inn i strmmen
		buffer.write((almanac.wn & 0xff00) >> 8);
		buffer.write(0);
		buffer.write(0);

		//DWORD0
		long e = Math.round(almanac.e / Math.pow(2, -21));					// Skriver dword0 inn i strmmen. dword 0 inneholdet e og
		buffer.write((int)(e & 0xff));
		buffer.write((int)(e & 0xff00) >> 8);
		buffer.write((1<<6) | (svid&0x3f));	// dataid = 01
		buffer.write(0);

		//DWORD1
		long i0 = Math.round((almanac.i0 - 0.3) / Math.pow(2, -19));
		buffer.write((int)(i0 & 0xff));
		buffer.write((int)(i0 & 0xff00) >> 8);
		buffer.write((int)(almanac.toa / Math.pow(2, 12)));
		buffer.write((almanac.i0>=0 ? 0 : 0xff));

		//DWORD2
		buffer.write((int)(almanac.health));
		long omegadot = Math.round(almanac.omegadot / Math.pow(2, -38));
		buffer.write((int)(omegadot & 0xff));
		buffer.write((int)(omegadot & 0xff00) >> 8);
		buffer.write((int)(omegadot & 0xff0000) >> 16);

		//DWORD3
		long roota = Math.round(almanac.roota / Math.pow(2, -11));
		buffer.write((int)(roota & 0xff));
		buffer.write((int)(roota & 0xff00) >> 8);
		buffer.write((int)(roota & 0xff0000) >> 16);
		buffer.write(0xff);

		//DWORD4
		long omega0 = Math.round(almanac.omega0 / Math.pow(2, -23));
		buffer.write((int)(omega0 & 0xff));
		buffer.write((int)(omega0 & 0xff00) >> 8);
		buffer.write((int)(omega0 & 0xff0000) >> 16);
		//buffer.write((int)(omega0 & 0xff000000) >> 24);
		buffer.write(almanac.omega0>=0 ? 0: 0xff);							// Skriver inn 0xff p frste bits hvis negativt

		//DWORD5
		long omega = Math.round(almanac.omega / Math.pow(2, -23));
		buffer.write((int)(omega & 0xff));
		buffer.write((int)(omega & 0xff00) >> 8);
		buffer.write((int)(omega & 0xff0000) >> 16);
		//buffer.write((int)(omega & 0xff000000) >> 24);
		buffer.write(almanac.omega>=0 ? 0: 0xff);							// Skriver inn 0xff p frste bits hvis negativt

		//DWORD6
		long m0 = Math.round(almanac.m0 / Math.pow(2, -23));
		buffer.write((int)(m0 & 0xff));
		buffer.write((int)(m0 & 0xff00) >> 8);
		buffer.write((int)(m0 & 0xff0000) >> 16);
		//buffer.write((int)(m0 & 0xff000000) >> 24);
		buffer.write(almanac.m0>=0 ? 0: 0xff);								// Skriver inn 0xff p frste bit hvis negativt


		//DWORD7
		long af0 = Math.round(almanac.af0 / Math.pow(2, -20));
		long af1 = Math.round(almanac.af1 / Math.pow(2, -38));
		//System.out.println("af0="+af0+", af1="+af1);
		buffer.write((int)(((af0 & 0x7) << 2) | ((af1 & 0x7) << 5)));
		buffer.write((int)(af1 & 0x7F8) >> 3);
		buffer.write((int)(af0 & 0x7F8) >> 3);
		buffer.write((almanac.af0>0 ? 0 : 0xff));							// Skriver inn 0xff p frste bit hvis negativt

		ByteArrayOutputStream msg = new ByteArrayOutputStream(48);			// Lager en ny outputstream
		// Header
		msg.write(0xb5);													// Skriver sync characters inn i ny outputstream
		msg.write(0x62);

		try {
			buffer.writeTo(msg);											// Skriver outputstreamen buffer inn i outputstreamen msg
		} catch (IOException e1) {											// Tar seg av exception
			e1.printStackTrace();
		}


		// Checksum
		int ck_a=0, ck_b=0;
// 		String sbBuffer = buffer.toString();  // her fr vi noen ganger et array som er en byte kortere, SJJ
		byte[] bbBuffer = buffer.toByteArray();								// Bruker innholdet i buffer da dette er klasse, id, lengde og nyttelasten
		for (int i=0; i<44; i++) {
//			ck_a = (ck_a + sbBuffer.charAt(i)) & 0xff;
			ck_a = (ck_a + bbBuffer[i]) & 0xff;								// Beregner checksum etter eksempel i UBX-hjelpefil
			ck_b = (ck_b + ck_a) & 0xff;
			//System.out.println("byte(" + i + ")="+(sbBuffer.charAt(i) & 0xff) + ", ck_a="+ck_a+", ck_b="+ck_b);
		}

		msg.write(ck_a);													// Skriver checksumen inn i outputstreamen
		msg.write(ck_b);

		// Print debug information
		if (DEBUG) {
			String raw = msg.toString();
			for (int i=0; i<raw.length(); i++) {
				String str = Integer.toHexString(raw.charAt(i) & 0xff);
				if (str.length() == 1) System.out.print("0");
				System.out.print(str+" ");
			}
			System.out.println();
		}

		return msg.toByteArray();											// Returnerer en nylaget byte array
	}

	public byte[] getAID_EPH(Ephemeris eph) {
		// For kommentarer se almanakk metoden
		ByteArrayOutputStream buffer = new ByteArrayOutputStream(104);

		// ID
		buffer.write(0x0b);
		buffer.write(0x31);

		// Length
		buffer.write(104);
		buffer.write(0);

		// SVID
		int svid = eph.prn+1;
		buffer.write(svid);
		buffer.write(0);
		buffer.write(0);
		buffer.write(0);

		// HOW
		buffer.write((int)(eph.tow >> 9));
		buffer.write((int)((eph.tow & 0x1fe) >> 1));
		buffer.write((int)((eph.tow & 0x01) << 7) | 0x4);
		buffer.write(0);

		// Subframe 1 Word 0
		buffer.write(eph.wn & 0xff);
		buffer.write((eph.wn & 0x300) >> 8);
		buffer.write((eph.health & 0x3F << 2) | (int)(eph.aodc & 0x300) >> 8);
		buffer.write(0);

		// Subframe 1 Word 1
		buffer.write(0);
		buffer.write(0);
		buffer.write(0);
		buffer.write(0);

		// Subframe 1 Word 2
		buffer.write(0);
		buffer.write(0);
		buffer.write(0);
		buffer.write(0);

		// Subframe 1 Word 3
		buffer.write(0);
		buffer.write(0);
		buffer.write(0);
		buffer.write(0);

		// Subframe 1 Word 4
		buffer.write((int)(Math.round(eph.tgd / Math.pow(2, -31)) & 0xff));
		buffer.write(0);
		buffer.write(0);
		buffer.write((eph.tgd>=0 ? 0 : 0xff));

		// Subframe 1 Word 5
		long toc = Math.round(eph.toc / Math.pow(2, 4));
		buffer.write((int)(toc & 0xff));
		buffer.write((int)(toc & 0xff00) >> 8);
		buffer.write((int)(eph.aodc & 0xff));
		buffer.write((eph.toc>=0 ? 0 : 0xff));

		// Subframe 1 Word 6
		long af1 = Math.round(eph.af1 / Math.pow(2, -43));
		buffer.write((int)(af1 & 0xff));
		buffer.write((int)(af1 & 0xff00) >> 8);
		buffer.write((int)(Math.round(eph.af2 / Math.pow(2, -55)) & 0xff));
		buffer.write((eph.af1>=0 ? 0 : 0xff));

		// Subframe 1 Word 7
		long af0 = Math.round(eph.af0 / Math.pow(2, -31));
		buffer.write((int)(af0 & 0xff));
		buffer.write((int)(af0 & 0xff00) >> 8);
		buffer.write((int)(af0 & 0x3f0000 >> 16));
		buffer.write((eph.af0>=0 ? 0 : 0xff));


		// Subframe 2 Word 0
		long crs = Math.round(eph.crs / Math.pow(2, -5));
		buffer.write((int)(crs & 0xff));
		buffer.write((int)(crs & 0xff00) >> 8);
		buffer.write((int)(eph.aode & 0xff));
		buffer.write((eph.crs>=0 ? 0 : 0xff));

		// Subframe 2 Word 1
		long m0 = Math.round(eph.m0 / Math.pow(2, -31));
		long deltan = Math.round(eph.deltan / Math.pow(2, -43));
		buffer.write((int)(m0 & 0xff000000) >> 24);
		buffer.write((int)(deltan & 0xff));
		buffer.write((int)(deltan & 0xff00) >> 8);
		buffer.write((eph.deltan>=0 ? 0 : 0xff));

		// Subframe 2 Word 2
		buffer.write((int)(m0 & 0xff));
		buffer.write((int)(m0 & 0xff00) >> 8);
		buffer.write((int)(m0 & 0xff0000) >> 16);
		buffer.write((eph.m0>=0 ? 0 : 0xff));

		// Subframe 2 Word 3
		long cuc = Math.round(eph.cuc / Math.pow(2, -29));
		long e = Math.round(eph.e / Math.pow(2, -33));
		buffer.write((int)(e & 0xff000000) >> 24);
		buffer.write((int)(cuc & 0xff));
		buffer.write((int)(cuc & 0xff00) >> 8);
		buffer.write((eph.cuc>=0 ? 0 : 0xff));

		// Subframe 2 Word 4
		buffer.write((int)(e & 0xff));
		buffer.write((int)(e & 0xff00) >> 8);
		buffer.write((int)(e & 0xff0000) >> 16);
		buffer.write((eph.e>=0 ? 0 : 0xff));

		// Subframe 2 Word 5
		long cus = Math.round(eph.cus / Math.pow(2, -29));
		long roota = Math.round(eph.roota / Math.pow(2, -19));
		buffer.write((int)(roota & 0xff000000) >> 24);
		buffer.write((int)(cus & 0xff));
		buffer.write((int)(cus & 0xff00) >> 8);
		buffer.write((eph.cus>=0 ? 0 : 0xff));

		// Subframe 2 Word 6
		buffer.write((int)(roota & 0xff));
		buffer.write((int)(roota & 0xff00) >> 8);
		buffer.write((int)(roota & 0xff0000) >> 16);
		buffer.write((eph.roota>=0 ? 0 : 0xff));

		// Subframe 2 Word 7
		long toe = Math.round(eph.toe / Math.pow(2, 4));
		buffer.write((eph.fit & 0x1)<<7);
		buffer.write((int)(toe & 0xff));
		buffer.write((int)(toe & 0xff00) >> 8);
		buffer.write((eph.toe>=0 ? 0 : 0xff));

		// Subframe 3 Word 0
		long cic = Math.round(eph.cic / Math.pow(2, -29));
		long omega0 = Math.round(eph.omega0 / Math.pow(2, -31));
		buffer.write((int)(omega0 & 0xff000000) >> 24);
		buffer.write((int)(cic & 0xff));
		buffer.write((int)(cic & 0xff00) >> 8);
		buffer.write((eph.cic>=0 ? 0 : 0xff));

		// Subframe 3 Word 1
		buffer.write((int)(omega0 & 0xff));
		buffer.write((int)(omega0 & 0xff00) >> 8);
		buffer.write((int)(omega0 & 0xff0000) >> 16);
		buffer.write((eph.omega0>=0 ? 0 : 0xff));

		// Subframe 3 Word 2
		long cis = Math.round(eph.cis / Math.pow(2, -29));
		long i0 = Math.round(eph.i0 / Math.pow(2, -31));
		buffer.write((int)(i0 & 0xff000000) >> 24);
		buffer.write((int)(cis & 0xff));
		buffer.write((int)(cis & 0xff00) >> 8);
		buffer.write((eph.cis>=0 ? 0 : 0xff));

		// Subframe 3 Word 3
		buffer.write((int)(i0 & 0xff));
		buffer.write((int)(i0 & 0xff00) >> 8);
		buffer.write((int)(i0 & 0xff0000) >> 16);
		buffer.write((eph.i0>=0 ? 0 : 0xff));

		// Subframe 3 Word 4
		long crc = Math.round(eph.crc / Math.pow(2, -5));
		long omega = Math.round(eph.omega / Math.pow(2, -31));
		buffer.write((int)(omega & 0xff000000) >> 24);
		buffer.write((int)(crc & 0xff));
		buffer.write((int)(crc & 0xff00) >> 8);
		buffer.write((eph.crc>=0 ? 0 : 0xff));

		// Subframe 3 Word 5
		buffer.write((int)(omega & 0xff));
		buffer.write((int)(omega & 0xff00) >> 8);
		buffer.write((int)(omega & 0xff0000) >> 16);
		buffer.write((eph.omega>=0 ? 0 : 0xff));

		// Subframe 3 Word 6
		long omegadot = Math.round(eph.omegadot / Math.pow(2, -43));
		buffer.write((int)(omegadot & 0xff));
		buffer.write((int)(omegadot & 0xff00) >> 8);
		buffer.write((int)(omegadot & 0xff0000) >> 16);
		buffer.write((eph.omegadot>=0 ? 0 : 0xff));

		// Subframe 3 Word 7
		long idot = Math.round(eph.idot / Math.pow(2, -43));
		buffer.write((int)(idot & 0x3f)<<2);
		buffer.write((int)(idot & 0x3fc0) >> 6);
		buffer.write((int)(eph.aode & 0xff));
		buffer.write((eph.idot>=0 ? 0 : 0xff));

		ByteArrayOutputStream msg = new ByteArrayOutputStream(108);

		// Header
		msg.write(0xb5);
		msg.write(0x62);

		try {
			buffer.writeTo(msg);
		} catch (IOException e1) {
			e1.printStackTrace();
		}

		// Checksum
		int ck_a=0, ck_b=0;
		// String sbBuffer = buffer.toString();
				byte[] bbBuffer = buffer.toByteArray();
		for (int i=0; i<104; i++) {
//			ck_a = (ck_a + sbBuffer.charAt(i)) & 0xff;
			ck_a = (ck_a + bbBuffer[i]) & 0xff;
			ck_b = (ck_b + ck_a) & 0xff;
			//System.out.println("byte(" + i + ")="+(sbBuffer.charAt(i) & 0xff) + ", ck_a="+ck_a+", ck_b="+ck_b);
		}

		msg.write(ck_a);
		msg.write(ck_b);

		// Print debug information
		if (DEBUG) {
			String raw = msg.toString();
			for (int i=0; i<raw.length(); i++) {
				String str = Integer.toHexString(raw.charAt(i) & 0xff);
				if (str.length() == 1) System.out.print("0");
				System.out.print(str+" ");
			}
			System.out.println();
		}

		return msg.toByteArray();
	}

	public byte[] getAID_INI() throws IOException {
		// Dette er en metode som lager en aid-ini melding. Denne meldingen er en initialiseringsmelding som sendes til gps modulen. den inneholder posisjon, tidspunkt og klokkedrift. i meldingen er alltid klokkedriften satt til 0 og merket som gyldig.

		int ecefx = 281399271;													// Posisjonens x koordinat i cm
		int ecefy = 51840398;													// Posisjonens y koordinat i cm
		int ecefz = 568136820;													// posisjonens z koordinat i cm
		long posacc = 1000;														// Nyaktigheten (standardavvik) i cm
		short wn = 1252;														// ukenummer

		ByteArrayOutputStream buffer = new ByteArrayOutputStream(52);			// lager en outputstream og legger parameterne inn i denne

		// ID
		buffer.write(0x0b);														// IDen til AID-INI meldingen
		buffer.write(0x01);

		// Length
		buffer.write(48);														// lengden til meldingen
		buffer.write(0);

		// X, Y, Z
		buffer.write(intToI4(ecefx));											// Posisjonen og nyaktigheten
		buffer.write(intToI4(ecefy));
		buffer.write(intToI4(ecefz));
		buffer.write(longToU4(posacc));

		// Timemark
		buffer.write(0x0);														// Timemark cfg satt til extint 0
		buffer.write(0x0);

		// WN
		buffer.write(intToU2(wn));												// Gjr om wn til U2 format fr det skrives inn

		// Time (ms)
		long tow = System.currentTimeMillis();									// Henter systemtid i ms og gjr om fra long til U4 format
		buffer.write(longToU4(tow));

		// Time (ns)
		buffer.write(intToI4(0));												// Setter ns i tiden til 0

		// Time accuracy (ms)
		buffer.write(intToI4(4));

		// Time accuracy (ns)
		buffer.write(intToI4(0));

		// Clockdrift
		buffer.write(intToI4(0));												// Som angitt i hjelpefilen settes klokkedriften til 0
		buffer.write(intToI4(0));

		// Flags
		buffer.write(intToI4(2));												// Nr man sette flaggene til 2 betyr dette at klokkedataene er gyldige

		ByteArrayOutputStream msg = new ByteArrayOutputStream(48);				// lager ny outputstream

		// Header
		msg.write(0xb5);														// Skriver inn sync characters inn i den nye strmmen
		msg.write(0x62);

		try {
			buffer.writeTo(msg);												// Skriver inn den gamle strmmen inn i den nye
		} catch (IOException e1) {
			e1.printStackTrace();
		}

		// Checksum
		int ck_a=0, ck_b=0;

//		String sbBuffer = buffer.toString();
				byte[] bbBuffer = buffer.toByteArray();							// Beregner checksummen etter metoden anngitt i hjelpefilen
		for (int i=0; i<52; i++) {
//			ck_a = (ck_a + sbBuffer.charAt(i)) & 0xff;
			ck_a = (ck_a + bbBuffer[i]) & 0xff;
			ck_b = (ck_b + ck_a) & 0xff;
			//System.out.println("byte(" + i + ")="+(sbBuffer.charAt(i) & 0xff) + ", ck_a="+ck_a+", ck_b="+ck_b);
		}

		msg.write(ck_a);														// Skriver checksummen inn i den nye strmmen
		msg.write(ck_b);

		if (DEBUG) {
			String raw = msg.toString("Cp1252");
			int c = 0;
			for (int i=0; i<raw.length(); i++) {
				c = raw.charAt(i) & 0xff;
				System.out.print((c <= 0xf) ? " 0" : " ");
				System.out.print(Integer.toHexString(c));
			}
			System.out.println();
		}

		return msg.toByteArray();												// Ved retur lages et nytt byte array og dataene i strmmen legges dit
	}

	private byte[] intToU2(int i) {												// Denne metoden lager en U2 av en int
		byte[] u2 = new byte[2];
		u2[0] = (byte)(i & 0x000000ff);
		u2[1] = (byte)((i >> 8) & 0x000000ff);
		return u2;
	}

	private byte[] intToI4(int i) {												// Denne metoden lager en I4 av en int
		byte[] i4 = new byte[4];
		i4[0] = (byte)(i & 0x000000ff);
		i4[1] = (byte)((i & 0x0000ff00) >> 8);
		i4[2] = (byte)((i & 0x00ff0000) >> 16);
		i4[3] = (byte)((i & 0xff000000) >> 24);
		return i4;
	}

	private byte[] longToU4(long l) {											// Denne metoden lager en U4 av en long
		byte[] u4 = new byte[4];
		u4[0] = (byte)(l & 0x00000000000000ff);
		u4[1] = (byte)((l >> 8) & 0x00000000000000ff);
		u4[2] = (byte)((l >> 16) & 0x00000000000000ff);
		u4[3] = (byte)((l >> 24) & 0x00000000000000ff);
		return u4;
	}
}
