package com.telenor.positionserver.gps;

import com.telenor.positionserver.PositionDevice;
import com.telenor.positionserver.Position;

import java.net.*;
import java.io.*;
import java.util.*;
import org.apache.log4j.Logger;

/**
 * Created by IntelliJ IDEA.
 * User: t533807
 * Date: 15.mar.2004
 * Time: 12:08:56
 * To change this template use Options | File Templates.
 */
public class GPSDevice extends Thread {
	private static Hashtable clients;

	public GPSDevice() {
		clients = new Hashtable();
		ConnectionHandler ch = new ConnectionHandler();
		ch.start();

	}

	public void run() {
		while (true) {
			Enumeration enum = clients.elements();
			while (enum.hasMoreElements()) {
				ClientData client = (ClientData) enum.nextElement();
				// For each client
				if (client.socket.isClosed()) {
					Logger.getLogger(GPSDevice.class).info("Client with id = "+client.id+" has closed the connection");
					synchronized (clients) {
						clients.remove(client.id);
					}
				} else {
					// Check for incomming data
					try {
						InputStreamReader in = new InputStreamReader(client.socket.getInputStream());
						if (in.ready()) {
							BufferedReader reader = new BufferedReader(in);
							String inputLine = reader.readLine();
							Logger.getLogger(GPSDevice.class).info("Msg from " + client.id + ": " + inputLine);
							client.nmea = inputLine;	// todo: Can have multiple requesters at the same time
						}
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	public static void shutdown() {
		Logger.getLogger(GPSDevice.class).info("Closing all open connections");
		Enumeration enum = clients.elements();
		while (enum.hasMoreElements()) {
			ClientData client = (ClientData) enum.nextElement();
			try {
				client.socket.close();
			} catch(IOException e) {
				e.printStackTrace();
			}
		}
	}

	public static void track(String id, int interval, int count) {
		ClientData client = (ClientData)clients.get(id);
		send(client.socket, "?TRS_"+interval+"_"+count);
	}

	public static Position findPosition(String userId, String id) {
		Logger.getLogger(GPSDevice.class).info("Find position for id=" + id);
		ClientData client = (ClientData)clients.get(id);
		if (client == null)
			return null;

		Position pos = new Position();
		pos.setEndUser(id);
		pos.setRequester(userId);
		pos.setPositionType("GPS");
		pos.setFound(false);
/*		synchronized (clients) {
			client.requests.put(userId, pos);
		}
*/
		// Send position request to client
		send(client.socket, "?LOC");

		// and wait for answer
		// todo: Return of position should not be handled here. Not good!
		for (int i=0; i<10; i++) {
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			if (client.nmea.length() > 0)
				return NMEADriver.parseNMEA(pos, client.nmea);
		}
		return pos;
	}

	private static void send(Socket client, String msg) {
		if (client != null) {
			try {
				PrintWriter out = new PrintWriter(client.getOutputStream(), true);
				out.println(msg);
			} catch (IOException e) {
				e.printStackTrace();
			}
		} else {
			Logger.getLogger(GPSDevice.class).error("Client is not connected!");
		}
	}

	private class ConnectionHandler extends Thread {
		ServerSocket server;
		public ConnectionHandler() {
			try {
				server = new ServerSocket(8001);
			} catch (IOException e) {
				Logger.getLogger(GPSDevice.class).error("Unable to create server on socket 8001");
			}
		}

		public void run() {
			while (true) {
				// Accept incomming connections forever
				try {
					Socket clientConnect = server.accept();
					BufferedReader in = new BufferedReader(new InputStreamReader(
                            clientConnect.getInputStream()));
					//PrintWriter out = new PrintWriter(clientConnect.getOutputStream(), true);
					String inputLine;
					if ((inputLine = in.readLine()) != null) {
						if (inputLine.charAt(0) != '!') {
							Logger.getLogger(GPSDevice.class).warn("Error in command from client. '" + inputLine + "'");
						} else {
							if (inputLine.startsWith("!IN")) {
								String id = inputLine.substring(4);
								Logger.getLogger(GPSDevice.class).info("Login from " + id + ". Connection from address: "
										  + clientConnect.getInetAddress() + ":" + clientConnect.getPort());
								ClientData client = new ClientData();
								client.id = id;
								client.socket = clientConnect;
								synchronized (clients) {
									clients.put(id, client);
								}
							}
						}
					}
					//in.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public String getDeviceInfo() {
		return "GPS from Mobilephone via GPRS";
	}

	public String getDeviceType() {
		return "GPS";
	}
}
